//---------------------------------------------------------------------------------------
//  Copyright (c) 2016 Firaxis Games, Inc. All rights reserved.
//---------------------------------------------------------------------------------------
class X2Ability_ConcussiveShot extends X2Ability;

var private name StunImpairingAbilityName;

static function array<X2DataTemplate> CreateTemplates()
{
	local array<X2DataTemplate> Templates;
	
	Templates.AddItem(ConcussiveShotAbility());
	Templates.AddItem(ConcussiveStun()); 
	Templates.AddItem(RogueSap());
	Templates.AddItem(Deadeye3Damage());
	//Templates.AddItem(RiposteShot2());
	
	return Templates;
}

static function X2AbilityTemplate ConcussiveShotAbility()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityCooldown                 Cooldown;
	local X2AbilityToHitCalc_StandardAim    ToHitCalc;
	local X2Condition_Visibility            TargetVisibilityCondition;
	local X2AbilityCost_Ammo                AmmoCost;
	local X2AbilityCost_ActionPoints        ActionPointCost;
	local X2Condition_UnitProperty          TargetPropertyCondition;
	local X2Condition_UnitProperty          ShooterPropertyCondition;
	local X2Effect_ImmediateAbilityActivation ImpairingAbilityEffect;
	local X2Effect_ApplyWeaponDamage        WeaponDamageEffect;
	local X2AbilityCharges                      Charges;
	local X2AbilityCost_Charges             ChargesCost;

	 

	`CREATE_X2ABILITY_TEMPLATE(Template, 'ConcussiveShot');



	Template.AdditionalAbilities.AddItem('Deadeye2Damage');
	Template.AdditionalAbilities.AddItem('ConcussiveStun');
	Template.AdditionalAbilities.AddItem('RogueSap');
	Template.AdditionalAbilities.AddItem('RiposteShot2');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_bringemon";
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.Hostility = eHostility_Offensive;
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_LIEUTENANT_PRIORITY;
	Template.AbilityConfirmSound = "TacticalUI_ActivateAbility";

	Template.TargetingMethod = class'X2TargetingMethod_OverTheShoulder';
	Template.bUsesFiringCamera = true;
	Template.CinescriptCameraType = "StandardGunFiring";

		//Don't trigger when the source is concealed   HERE
//	SourceNotConcealedCondition = new class'X2Condition_UnitProperty';
//	SourceNotConcealedCondition.ExcludeConcealed = true;
//	Template.AbilityShooterConditions.AddItem(SourceNotConcealedCondition);

	Cooldown = new class'X2AbilityCooldown';
	Cooldown.iNumTurns = 2;
	Template.AbilityCooldown = Cooldown;

	//Template.AbilityCosts.AddItem(new class'X2AbilityCost_Charges');
	Charges = new class'X2AbilityCharges';
	Charges.InitialCharges = 3; 
	Template.AbilityCharges = Charges;

	ChargesCost = new class'X2AbilityCost_Charges';
	ChargesCost.NumCharges = 1;
	ChargesCost.SharedAbilityCharges.AddItem('RogueSap');
	Template.AbilityCosts.AddItem(ChargesCost);

	ToHitCalc = new class'X2AbilityToHitCalc_StandardAim';
	ToHitCalc.FinalMultiplier = 1;
	ToHitCalc.bAllowCrit = false;
	Template.AbilityToHitCalc = ToHitCalc;

	AmmoCost = new class'X2AbilityCost_Ammo';
	AmmoCost.iAmmo = 1;
	Template.AbilityCosts.AddItem(AmmoCost);

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

	Template.AbilityTargetStyle = default.SimpleSingleTarget;
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	TargetVisibilityCondition = new class'X2Condition_Visibility';
	TargetVisibilityCondition.bRequireGameplayVisible = true;
	TargetVisibilityCondition.bAllowSquadsight = true;
	Template.AbilityTargetConditions.AddItem(TargetVisibilityCondition);
	Template.AbilityTargetConditions.AddItem(default.LivingHostileTargetProperty);

	//  Put holo target effect first because if the target dies from this shot, it will be too late to notify the effect.
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.HoloTargetEffect());
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.ShredderDamageEffect());

	Template.bAllowAmmoEffects = true;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;

	Template.bCrossClassEligible = false;

	// Target Conditions
	//
	TargetPropertyCondition = new class'X2Condition_UnitProperty';	
	TargetPropertyCondition.ExcludeDead = true;                     //Can't target dead
	TargetPropertyCondition.ExcludeFriendlyToSource = true;         //Can't target friendlies
	Template.AbilityTargetConditions.AddItem(TargetPropertyCondition);

	// Shooter Conditions
	//
	ShooterPropertyCondition = new class'X2Condition_UnitProperty';	
	ShooterPropertyCondition.ExcludeDead = true;                    //Can't shoot while dead
	Template.AbilityShooterConditions.AddItem(ShooterPropertyCondition);

	//Impairing effects need to come before the damage. This is needed for proper visualization ordering.
	//Effect on a successful melee attack is triggering the Apply Impairing Effect Ability
	ImpairingAbilityEffect = new class 'X2Effect_ImmediateAbilityActivation';
	ImpairingAbilityEffect.BuildPersistentEffect(1, false, true, , eGameRule_PlayerTurnBegin);
	ImpairingAbilityEffect.EffectName = 'ImmediateStunImpair';
	ImpairingAbilityEffect.AbilityName = 'ConcussiveStun';
	ImpairingAbilityEffect.bRemoveWhenTargetDies = true;
	ImpairingAbilityEffect.VisualizationFn = ImpairingAbilityEffectTriggeredVisualization;
	Template.AddTargetEffect(ImpairingAbilityEffect);

	// Damage Effect
	//
	//WeaponDamageEffect = new class'X2Effect_ApplyWeaponDamage';
	WeaponDamageEffect.DamageTypes.AddItem('Electrical');
	Template.AddTargetEffect(WeaponDamageEffect);

	Template.ActivationSpeech = 'DisablingShot'; 
 
	return Template;
}

static function X2DataTemplate ConcussiveStun()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityToHitCalc_StatCheck_UnitVsUnit    StatContest;
	local X2AbilityTarget_Single            SingleTarget;
	local X2Effect_Persistent               DisorientedEffect;
	local X2Effect_Persistent               DisorientedEffect3;	
	local X2Effect_Stunned				    StunnedEffect;
	local X2Effect_Persistent               UnconsciousEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'ConcussiveStun');
	Template.AdditionalAbilities.AddItem('Deadeye2Damage');
	Template.ConcealmentRule = eConceal_Always;
	Template.bDontDisplayInAbilitySummary = true;
	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;

	SingleTarget = new class'X2AbilityTarget_Single';
	SingleTarget.OnlyIncludeTargetsInsideWeaponRange = true;
	Template.AbilityTargetStyle = SingleTarget;

	Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');      //  ability is activated by another ability that hits

	// Target Conditions
	//
	Template.AbilityTargetConditions.AddItem(default.LivingTargetUnitOnlyProperty);
	Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);

	// Shooter Conditions
	//
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);

	Template.AddShooterEffectExclusions();

	// This will be a stat contest
	StatContest = new class'X2AbilityToHitCalc_StatCheck_UnitVsUnit';
	StatContest.AttackerStat = estat_Will;
	//	StatContest.AttackerStat = eStat_Offense;
	Template.AbilityToHitCalc = StatContest;

	// On hit effects
	//  Stunned effect for 1 or 2 unblocked hit
	DisorientedEffect = class'X2StatusEffects'.static.CreateDisorientedStatusEffect();
	DisorientedEffect.BuildPersistentEffect(2,, false,,eGameRule_PlayerTurnBegin);
	DisorientedEffect.MinStatContestResult = 1;
	DisorientedEffect.MaxStatContestResult = 2;
	DisorientedEffect.bRemoveWhenSourceDies = false;
	Template.AddTargetEffect(DisorientedEffect);

	//  Stunned effect for 3 or 4 unblocked hit
	StunnedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	StunnedEffect.MinStatContestResult = 3;
	StunnedEffect.MaxStatContestResult = 4;
	StunnedEffect.bRemoveWhenSourceDies = false;
	Template.AddTargetEffect(StunnedEffect);

	UnconsciousEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	UnconsciousEffect.MinStatContestResult = 5;
	UnconsciousEffect.MaxStatContestResult = 0;
	UnconsciousEffect.bRemoveWhenSourceDies = false;
	Template.AddTargetEffect(UnconsciousEffect);
	//Unconscious effect for 5 unblocked hits

	DisorientedEffect3 = class'X2StatusEffects'.static.CreateDisorientedStatusEffect();
	DisorientedEffect3.BuildPersistentEffect(3,, false,,eGameRule_PlayerTurnBegin);
	DisorientedEffect3.MinStatContestResult = 5;
	DisorientedEffect3.MaxStatContestResult = 0;
	DisorientedEffect3.bRemoveWhenSourceDies = false;
	Template.AddTargetEffect(DisorientedEffect3);

	
	// UnconsciousEffect = class'X2StatusEffects'.static.CreateUnconsciousStatusEffect(true);
	// UnconsciousEffect.MinStatContestResult = 5;
	// UnconsciousEffect.MaxStatContestResult = 0;
	// UnconsciousEffect.bRemoveWhenSourceDies = false;
	// Template.AddTargetEffect(UnconsciousEffect);

	Template.bSkipPerkActivationActions = true;
	Template.bSkipFireAction = true;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = StunLancerImpairing_BuildVisualization;

	return Template;
}

simulated function StunLancerImpairing_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory History;
	local XComGameStateContext_Ability  Context;
	local StateObjectReference InteractingUnitRef;
	local VisualizationTrack EmptyTrack;
	local VisualizationTrack BuildTrack;

	History = `XCOMHISTORY;

	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());
	InteractingUnitRef = Context.InputContext.SourceObject;

	//Configure the visualization track for the shooter
	//****************************************************************************************
	BuildTrack = EmptyTrack;
	BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	BuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
	BuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

	class'X2Action_AbilityPerkStart'.static.AddToVisualizationTrack(BuildTrack, Context);
	class'X2Action_AbilityPerkEnd'.static.AddToVisualizationTrack(BuildTrack, Context);

	OutVisualizationTracks.AddItem(BuildTrack);
}

static function ImpairingAbilityEffectTriggeredVisualization(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	local XComGameStateContext_Ability AbilityContext;
	local XComGameStateContext Context;
	local XComGameStateContext_Ability TestAbilityContext;
	local int i, j;
	local XComGameStateHistory History;
	local bool bAbilityWasSuccess;
	local X2AbilityTemplate AbilityTemplate;
	local X2VisualizerInterface TargetVisualizerInterface;

	if( (EffectApplyResult != 'AA_Success') || (XComGameState_Unit(BuildTrack.StateObject_NewState) == none) )
	{
		return;
	}

	Context = VisualizeGameState.GetContext();
	AbilityContext = XComGameStateContext_Ability(Context);

	if( AbilityContext.EventChainStartIndex != 0 )
	{
		History = `XCOMHISTORY;

		// This GameState is part of a chain, which means there may be a stun to the target
		for( i = AbilityContext.EventChainStartIndex; !Context.bLastEventInChain; ++i )
		{
			Context = History.GetGameStateFromHistory(i).GetContext();

			TestAbilityContext = XComGameStateContext_Ability(Context);
			bAbilityWasSuccess = (TestAbilityContext != none) && class'XComGameStateContext_Ability'.static.IsHitResultHit(TestAbilityContext.ResultContext.HitResult);

			if( bAbilityWasSuccess &&
				TestAbilityContext.InputContext.AbilityTemplateName == 'ConcussiveStun' &&
				TestAbilityContext.InputContext.SourceObject.ObjectID == AbilityContext.InputContext.SourceObject.ObjectID &&
				TestAbilityContext.InputContext.PrimaryTarget.ObjectID == AbilityContext.InputContext.PrimaryTarget.ObjectID )
			{
				// The Melee Impairing Ability has been found with the same source and target
				// Move that ability's visualization forward to this track
				AbilityTemplate = class'XComGameState_Ability'.static.GetMyTemplateManager().FindAbilityTemplate(TestAbilityContext.InputContext.AbilityTemplateName);

				for( j = 0; j < AbilityTemplate.AbilityTargetEffects.Length; ++j )
				{
					AbilityTemplate.AbilityTargetEffects[j].AddX2ActionsForVisualization(Context.AssociatedState, BuildTrack, TestAbilityContext.FindTargetEffectApplyResult(AbilityTemplate.AbilityTargetEffects[j]));
				}

				TargetVisualizerInterface = X2VisualizerInterface(BuildTrack.TrackActor);
				if (TargetVisualizerInterface != none)
				{
					TargetVisualizerInterface.BuildAbilityEffectsVisualization(Context.AssociatedState, BuildTrack);
				}
			}
		}
	}
}

////defaultproperties
////{
	////StunImpairingAbilityName="StunImpairingAbility"
////}
//
static function X2AbilityTemplate RogueSap()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityCooldown                 Cooldown;
	local X2AbilityToHitCalc_StandardAim    ToHitCalc;
	local X2Condition_Visibility            TargetVisibilityCondition;
	local X2AbilityCost_Ammo                AmmoCost;
	local X2AbilityCost_ActionPoints        ActionPointCost;
	local X2Condition_UnitProperty          TargetPropertyCondition;
	local X2Condition_UnitProperty          ShooterPropertyCondition;
	local X2Effect_ImmediateAbilityActivation ImpairingAbilityEffect;
	local X2Effect_ApplyWeaponDamage        WeaponDamageEffect;
	local X2AbilityCost_Charges             ChargesCost;
	local X2AbilityCharges              Charges;
	local X2Condition_UnitProperty          ShooterCondition;	 //here
//	local X2Condition_UnitProperty          SourceNotConcealedCondition;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'RogueSap');

	Template.ConcealmentRule = eConceal_Always;

	Template.AdditionalAbilities.AddItem('RiposteShot2');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_sectoid_mindspin";
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_ShowIfAvailable;
	Template.Hostility = eHostility_Offensive;
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_LIEUTENANT_PRIORITY;
	Template.AbilityConfirmSound = "TacticalUI_ActivateAbility";

	Template.TargetingMethod = class'X2TargetingMethod_OverTheShoulder';
	Template.bUsesFiringCamera = true;
	Template.CinescriptCameraType = "StandardGunFiring";

		//Don't trigger when the source is concealed   HERE
//	SourceNotConcealedCondition = new class'X2Condition_UnitProperty';
//	SourceNotConcealedCondition.ExcludeConcealed = true;
//	Template.AbilityShooterConditions.AddItem(SourceNotConcealedCondition);

	Cooldown = new class'X2AbilityCooldown';
	Cooldown.iNumTurns = 2;
	Template.AbilityCooldown = Cooldown;


	//Template.AbilityCosts.AddItem(new class'X2AbilityCost_Charges');
	//Charges = new class'X2AbilityCharges';
	//Charges.InitialCharges = 3; 
	//Template.AbilityCharges = Charges;
	
	Charges = new class'X2AbilityCharges';
	Charges.InitialCharges = 4;
	Template.AbilityCharges = Charges;

	//Template.AbilityCharges = new class'X2AbilityCost_Charges';  //works without this?
	ChargesCost = new class'X2AbilityCost_Charges';
	ChargesCost.NumCharges = 2;
	ChargesCost.SharedAbilityCharges.AddItem('ConcussiveShot');
	Template.AbilityCosts.AddItem(ChargesCost);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);	
	ShooterCondition = new class'X2Condition_UnitProperty';
	ShooterCondition.IsUnspotted = true;  //fix this here needs to only be activateable while concealed.
	Template.AbilityShooterConditions.AddItem(ShooterCondition);

	 


	ToHitCalc = new class'X2AbilityToHitCalc_StandardAim';
	ToHitCalc.FinalMultiplier = 1;
	Template.AbilityToHitCalc = ToHitCalc;

	AmmoCost = new class'X2AbilityCost_Ammo';
	AmmoCost.iAmmo = 1;
	Template.AbilityCosts.AddItem(AmmoCost);

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

	Template.AbilityTargetStyle = default.SimpleSingleTarget;
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	TargetVisibilityCondition = new class'X2Condition_Visibility';
	TargetVisibilityCondition.bRequireGameplayVisible = true;
	TargetVisibilityCondition.bAllowSquadsight = true;
	Template.AbilityTargetConditions.AddItem(TargetVisibilityCondition);
	Template.AbilityTargetConditions.AddItem(default.LivingHostileTargetProperty);

	//  Put holo target effect first because if the target dies from this shot, it will be too late to notify the effect.
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.HoloTargetEffect());
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.ShredderDamageEffect());

	Template.bAllowAmmoEffects = true;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;

	Template.bCrossClassEligible = false;

	// Target Conditions
	//
	TargetPropertyCondition = new class'X2Condition_UnitProperty';	
	TargetPropertyCondition.ExcludeDead = true;                     //Can't target dead
	TargetPropertyCondition.ExcludeFriendlyToSource = true;         //Can't target friendlies
	Template.AbilityTargetConditions.AddItem(TargetPropertyCondition);

	// Shooter Conditions
	//
	ShooterPropertyCondition = new class'X2Condition_UnitProperty';	
	ShooterPropertyCondition.ExcludeDead = true;                    //Can't shoot while dead
	Template.AbilityShooterConditions.AddItem(ShooterPropertyCondition);

	//Impairing effects need to come before the damage. This is needed for proper visualization ordering.
	//Effect on a successful melee attack is triggering the Apply Impairing Effect Ability
	ImpairingAbilityEffect = new class 'X2Effect_ImmediateAbilityActivation';
	ImpairingAbilityEffect.BuildPersistentEffect(1, false, true, , eGameRule_PlayerTurnBegin);
	ImpairingAbilityEffect.EffectName = 'ImmediateStunImpair';
	ImpairingAbilityEffect.AbilityName = 'ConcussiveStun';
	ImpairingAbilityEffect.bRemoveWhenTargetDies = true;
	ImpairingAbilityEffect.VisualizationFn = ImpairingAbilityEffectTriggeredVisualization;
	Template.AddTargetEffect(ImpairingAbilityEffect);

	// Damage Effect
	//
	//WeaponDamageEffect = new class'X2Effect_ApplyWeaponDamage';
//	WeaponDamageEffect.DamageTypes.AddItem('Electrical');
//	Template.AddTargetEffect(WeaponDamageEffect);

 
	Template.ActivationSpeech = 'DisablingShot'; 

	return Template;
}


//static function X2DataTemplate RiposteShot2()
//{
	//local X2AbilityTemplate                 Template;
	//local X2AbilityToHitCalc_StatCheck_UnitVsUnit    StatContest;
	//local X2AbilityTarget_Single            SingleTarget;
	//local X2Effect_Stunned					DisorientedEffect;
 //
	//local X2Effect_Stunned				    StunnedEffect;
	//local X2Effect_Stunned					UnconsciousEffect;
//
		//`CREATE_X2ABILITY_TEMPLATE(Template, 'RiposteShot2');
//
	//Template.bDontDisplayInAbilitySummary = true;
	//Template.AbilitySourceName = 'eAbilitySource_Standard';
	//Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
//
	//SingleTarget = new class'X2AbilityTarget_Single';
	//SingleTarget.OnlyIncludeTargetsInsideWeaponRange = true;
	//Template.AbilityTargetStyle = SingleTarget;
//
	//Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');      //  ability is activated by another ability that hits
//
	//// Target Conditions
	////
	//Template.AbilityTargetConditions.AddItem(default.LivingTargetUnitOnlyProperty);
	//Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);
//
	//// Shooter Conditions
	////
	//Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
//
	//Template.AddShooterEffectExclusions();
//
	//// This will be a stat contest
	//StatContest = new class'X2AbilityToHitCalc_StatCheck_UnitVsUnit';
	//StatContest.AttackerStat = estat_Will;
	////	StatContest.AttackerStat = eStat_Offense;
	//Template.AbilityToHitCalc = StatContest;
//
	//// On hit effects
	////  Stunned effect for 1 or 2 unblocked hit
	 //
	//DisorientedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	//DisorientedEffect.MinStatContestResult = 0;
	//DisorientedEffect.MaxStatContestResult = 2;
	//DisorientedEffect.bRemoveWhenSourceDies = false;
	//Template.AddTargetEffect(DisorientedEffect);
//
 	////  Stunned effect for 3 or 4 unblocked hit
	//StunnedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	//StunnedEffect.MinStatContestResult = 3;
	//StunnedEffect.MaxStatContestResult = 4;
	//StunnedEffect.bRemoveWhenSourceDies = false;
	//Template.AddTargetEffect(StunnedEffect);
//
 	//UnconsciousEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	//UnconsciousEffect.MinStatContestResult = 5;
	//UnconsciousEffect.MaxStatContestResult = 0;
	//UnconsciousEffect.bRemoveWhenSourceDies = false;
	//Template.AddTargetEffect(UnconsciousEffect);
 //
//
	//Template.bSkipPerkActivationActions = true;
	//Template.bSkipFireAction = true;
	//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//Template.BuildVisualizationFn = StunLancerImpairing_BuildVisualization;
//
	//return Template;
//}
//
 //
 //
//

static function X2AbilityTemplate Deadeye3Damage()
{
	local X2AbilityTemplate						Template;
	local X2Effect_Deadeye2Damage                DamageEffect;
 
	// Icon Properties
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Deadeye3Damage');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_momentum";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	DamageEffect = new class'X2Effect_Deadeye2Damage';
 	DamageEffect.DamageMultiplier = 0.50f;		
	DamageEffect.BuildPersistentEffect(1, true, false, false);
	DamageEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, false,,Template.AbilitySourceName);
	Template.AddTargetEffect(DamageEffect);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//  NOTE: No visualization on purpose!

	return Template;
}