//---------------------------------------------------------------------------------------
//  FILE:    X2Ability_RogueAbilitySet.uc
//  AUTHOR:  Andrew Lotz  --  02/07/2015
//  PURPOSE: Defines all Rogue Based Class Abilities
//           
//---------------------------------------------------------------------------------------
//  Copyright (c) 2016 Firaxis Games, Inc. All rights reserved.
//---------------------------------------------------------------------------------------
//extends X2Ability
class X2Ability_RogueAbilitySet extends X2Ability_SharpshooterAbilitySet
	dependson (XComGameStateContext_Ability) config(RogueAbilities);
	 
//var config int killzone_COOLDOWN;
var Name killzoneReserveType;           //  Type of action point to reserve for KillZone ability.
var private name StunImpairingAbilityName;
static function array<X2DataTemplate> CreateTemplates()
{
	local array<X2DataTemplate> Templates;
	
	Templates.AddItem(AddHuntersInstinctAbility2());
//	Templates.AddItem(AddHuntersInstinctAbility3());
	Templates.AddItem(BornOfShadows()); 
	Templates.AddItem(AddkernEyeAbility());
	Templates.AddItem(RogueAmbush());
	Templates.AddItem(RogueAmbushShot()); 
	Templates.AddItem(Deadeye2Ability());
	Templates.AddItem(Deadeye2Damage());
	Templates.AddItem(Untouchable2());
	Templates.AddItem(AddKnockoutSelfAbility2());
	Templates.AddItem(Stasis2());
	Templates.AddItem(Sustain2());
	Templates.AddItem(SustainTriggered2());
	Templates.AddItem(Faceoff2());
	Templates.AddItem(RapidFire3());
	Templates.AddItem(IsmsLowProfile());
	Templates.AddItem(RogueSpotter());
	Templates.AddItem(RogueStealthBonus());
    Templates.AddItem(ReturnFire3());

	Templates.AddItem(ConcussiveShot2Ability());
	Templates.AddItem(RiposteShot());
//	Templates.AddItem(RiposteShot6());

	
	Templates.AddItem(IsmsSelfSufficient());
	Templates.AddItem(IsmsSelfSufficient2Trigger());
	Templates.AddItem(IsmsSelfSufficient3Trigger());
	Templates.AddItem(IsmsSelfSufficient4Trigger());
	Templates.AddItem(IsmsSelfSufficient5Trigger());
	Templates.AddItem(IsmsSelfSufficientWill());
	//Templates.AddItem(IsmsTasteOfDeath());
	
	//Templates.AddItem(IsmsSoulCollector());
	Templates.AddItem(IsmsSoulCollector2());	
	Templates.AddItem(IsmsSoulCollector3());	

	//Templates.AddItem(IsmsTasteOfDeath());
	Templates.AddItem(IsmsTODStart());
	Templates.AddItem(IsmsOpenerStance());
	Templates.AddItem(IsmsCloserStance());
	Templates.AddItem(IsmsTODCooldown());

	Templates.AddItem(IsmsNBK());
	Templates.AddItem(IsmsNBK2());
 
//	Templates.AddItem(IsmsReducedThreat());
// 	Templates.AddItem(IsmsRemoveThreat());

    //Templates.AddItem(ReturnFire4());
//
	//Templates.AddItem(ConcussiveShot3Ability());



//	Templates.AddItem(IsmsInfiltrator());

	
	return Templates;
}

//******** Hunter's Instinct Ability **********
static function X2AbilityTemplate AddHuntersInstinctAbility2()
{
	local X2AbilityTemplate						Template;
	local X2Effect_RogueBlindside        DamageModifier;
//	local X2Effect_PersistentStatChange     PersistentStatChangeEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'HuntersInstinct2');

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_hunter";

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);
	
	//PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	//PersistentStatChangeEffect.AddPersistentStatChange(eStat_Mobility, 0);
	//Template.AddTargetEffect(PersistentStatChangeEffect);

	DamageModifier = new class'X2Effect_RogueBlindside';
//	DamageModifier.BonusDamage = 1;
	DamageModifier.BuildPersistentEffect(1, true, true, true);
	DamageModifier.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, true,,Template.AbilitySourceName);
//	DamageModifier.SetDisplayInfo(ePerkBuff_Passive, "Blinside", 
//		"Increases flanking damage by" @ (DamageModifier.BonusDamage) $ ".", Template.IconImage, true);
//JUST WRITE A DUMMY PERSISTENT EFFECT INSIDE BLINDSIDE

	Template.AddTargetEffect(DamageModifier);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//  NOTE: No visualization on purpose!


	return Template;
}

// ********************BornOfShadows********************


static function X2AbilityTemplate BornOfShadows()
{
	local X2AbilityTemplate						Template;
	local X2Effect_ToHitModifier                Effect;
	local X2Condition_Visibility                VisCondition;


	`CREATE_X2ABILITY_TEMPLATE(Template, 'BornOfShadows');

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_Stealth";
 

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	Effect = new class'X2Effect_ToHitModifier';
	Effect.EffectName = 'BornOfShadows';
	Effect.DuplicateResponse = eDupe_Ignore;
	Effect.BuildPersistentEffect(1, true, false);
	Effect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	Effect.AddEffectHitModifier(eHit_Success, 10, Template.LocFriendlyName);
	Effect.AddEffectHitModifier(eHit_Crit, 10, Template.LocFriendlyName);
	VisCondition = new class'X2Condition_Visibility';
	VisCondition.bExcludeGameplayVisible = true;
	Effect.ToHitConditions.AddItem(VisCondition);
	Template.AddTargetEffect(Effect);




	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//  NOTE: No visualization on purpose!

	return Template;
}

// ********************RoguesToolKit********************

//******** kerneye ability (gts) **********
static function x2abilitytemplate addkerneyeability()
{
	//local x2abilitytemplate						Template;
	//local x2abilitytargetstyle                  targetstyle;
	//local x2abilitytrigger						trigger;
//	local x2effect_tohitmodifier                tohitmodifier;
	local X2AbilityTemplate						Template;
	local x2effect_persistentstatchange     persistentstatchangeeffect;
//	local x2effect_huntersinstinctdamage        damagemodifier;

	//`create_x2ability_template(template, 'kerneye');
	//template.iconimage = "img:///uilibrary_perkicons.uiperk_hithurts";
//
	template.abilitysourcename = 'eabilitysource_perk';
	template.eabilityiconbehaviorhud = eabilityiconbehavior_nevershow;
	template.hostility = ehostility_neutral;
//
	//template.abilitytohitcalc = default.deadeye;
//
	//targetstyle = new class'x2abilitytarget_self';
	//template.abilitytargetstyle = targetstyle;
//
	//trigger = new class'x2abilitytrigger_unitpostbeginplay';
	//template.abilitytriggers.additem(trigger);

	
	persistentstatchangeeffect = new class'x2effect_persistentstatchange';
	persistentstatchangeeffect.addpersistentstatchange(estat_Hacking, 40);
	//persistentstatchangeeffect.addpersistentstatchange(estat_mobility, 2);
	//persistentstatchangeeffect.addpersistentstatchange(estat_flankingaimbonus, 5);
	//persistentstatchangeeffect.addpersistentstatchange(estat_armorpiercing, 1);
	//persistentstatchangeeffect.addpersistentstatchange(estat_defense, 3);
	////persistentstatchangeeffect.addpersistentstatchange(estat_detectionmodifier, .2f);
	template.addshootereffect(persistentstatchangeeffect);
	
	
 
	Template.SetUIStatMarkup(class'XLocalizedData'.default.TechBonusLabel, eStat_Hacking, 40, true);


	Template = PurePassive('kernEye', "img:///UILibrary_PerkIcons.UIPerk_holdtheline");
	Template.AdditionalAbilities.AddItem('LightningReflexes'); 
	Template.AdditionalAbilities.AddItem('IsmsKernEyeBonus'); 
	
	//Template.AdditionalAbilities.AddItem('Implacable'); 
	
	//template.buildnewgamestatefn = typicalability_buildgamestate;
	//  note: no visualization on purpose!

	return template;
}

 

// ***************************Killzone*************************

static function X2AbilityTemplate RogueAmbush()
{
 
    local X2AbilityTemplate				 Template;
    local X2AbilityCooldown				 Cooldown;
    local X2AbilityCost_Ammo			 AmmoCost;
    local X2AbilityCost_ActionPoints	 ActionPointCost;
    local X2AbilityTarget_Cursor		 CursorTarget;
    local X2AbilityMultiTarget_Cone		 ConeMultiTarget;
    local X2Effect_ReserveActionPoints		ReservePointsEffect;
    local X2Effect_MarkValidActivationTiles MarkTilesEffect;
    local X2Condition_UnitEffects           SuppressedCondition;
//  local X2AbilityCharges                      Charges;
 
    `CREATE_X2ABILITY_TEMPLATE(Template, 'RogueAmbush');

	Template.ConcealmentRule = eConceal_Always;
 
    AmmoCost = new class'X2AbilityCost_Ammo';
    AmmoCost.iAmmo = 1;
    AmmoCost.bFreeCost = true;
    Template.AbilityCosts.AddItem(AmmoCost);
 
    ActionPointCost = new class'X2AbilityCost_ActionPoints';
    ActionPointCost.iNumPoints = 1;
    ActionPointCost.bConsumeAllPoints = false;   //  this will guarantee the unit has at least 1 action point
    ActionPointCost.bFreeCost = true;           //  ReserveActionPoints effect will take all action points away
    Template.AbilityCosts.AddItem(ActionPointCost);
 
    Template.AbilityToHitCalc = default.DeadEye;
 
    Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
    Template.AddShooterEffectExclusions();
    SuppressedCondition = new class'X2Condition_UnitEffects';
    SuppressedCondition.AddExcludeEffect(class'X2Effect_Suppression'.default.EffectName, 'AA_UnitIsSuppressed');
    Template.AbilityShooterConditions.AddItem(SuppressedCondition);
 
    Cooldown = new class'X2AbilityCooldown';
    Cooldown.iNumTurns = 2;
    Template.AbilityCooldown = Cooldown;
    //Template.AbilityCosts.AddItem(new class'X2AbilityCost_Charges');
    //Charges = new class'X2AbilityCharges';
    //Charges.InitialCharges = 2;
    //Template.AbilityCharges = Charges;
 
    CursorTarget = new class'X2AbilityTarget_Cursor';
    CursorTarget.bRestrictToWeaponRange = true;
    Template.AbilityTargetStyle = CursorTarget;
 
    ConeMultiTarget = new class'X2AbilityMultiTarget_Cone';
    ConeMultiTarget.bUseWeaponRadius = true;
    ConeMultiTarget.ConeEndDiameter = 3.9* class'XComWorldData'.const.WORLD_StepSize;  //LOOK HERE
    ConeMultiTarget.ConeLength = 10 * class'XComWorldData'.const.WORLD_StepSize;		//LOOK HERE
    Template.AbilityMultiTargetStyle = ConeMultiTarget;
 
    Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);
 
    ReservePointsEffect = new class'X2Effect_ReserveActionPoints';
    ReservePointsEffect.ReserveType = 'RogueAmbush';
    ReservePointsEffect.NumPoints = 10;  //# of shots in overwatch
    Template.AddShooterEffect(ReservePointsEffect);
 
    // Call var int m_iMovePipsTouched  from dependson(XComGameState_Unit);
 
    MarkTilesEffect = new class'X2Effect_MarkValidActivationTiles';
    MarkTilesEffect.AbilityToMark = 'RogueAmbushShot';    //here
    Template.AddShooterEffect(MarkTilesEffect);
 
    Template.AdditionalAbilities.AddItem('RogueAmbushShot');  
    Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
    Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
    Template.bSkipFireAction = true;
    Template.bShowActivation = true;
 
    Template.AbilitySourceName = 'eAbilitySource_Perk';
    Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_AlwaysShow;
    Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_killzone";
    Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_MAJOR_PRIORITY;
    Template.bDisplayInUITooltip = false;
    Template.bDisplayInUITacticalText = false;
    Template.Hostility = eHostility_Neutral;
    Template.AbilityConfirmSound = "Unreal2DSounds_OverWatch";
 
    Template.ActivationSpeech = 'KillZone';
 	


    Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
    Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
    Template.TargetingMethod = class'X2TargetingMethod_Cone';
 
    Template.bCrossClassEligible = false;
 
    return Template;
}
 
//     function bool AllowCritOverride() { return true; }   // test
 
static function X2AbilityTemplate RogueAmbushShot()
{
    local X2AbilityTemplate                 Template;
    local X2AbilityCost_Ammo                AmmoCost;
    local X2AbilityCost_ReserveActionPoints ReserveActionPointCost;
    local X2AbilityToHitCalc_StandardAim    StandardAim;
    local X2Condition_AbilityProperty       AbilityCondition;
    local X2AbilityTarget_Single            SingleTarget;
    local X2AbilityTrigger_Event            Trigger;
    local X2Effect_Persistent               KillZoneEffect;
    local X2Condition_UnitEffectsWithAbilitySource  KillZoneCondition;
    local X2Condition_Visibility            TargetVisibilityCondition;
 

    `CREATE_X2ABILITY_TEMPLATE(Template, 'RogueAmbushShot');
 
 	Template.ConcealmentRule = eConceal_Always;

    AmmoCost = new class'X2AbilityCost_Ammo';
    AmmoCost.iAmmo = 1;
    Template.AbilityCosts.AddItem(AmmoCost);
   
//    Template.AdditionalAbilities.AddItem('DeadeyeDamage');    //testing
   
    ReserveActionPointCost = new class'X2AbilityCost_ReserveActionPoints';
    ReserveActionPointCost.iNumPoints = 1;              //here?
    ReserveActionPointCost.bFreeCost = false;
    ReserveActionPointCost.AllowedTypes.AddItem('RogueAmbush');                
    Template.AbilityCosts.AddItem(ReserveActionPointCost);
 
    StandardAim = new class'X2AbilityToHitCalc_StandardAim';
    StandardAim.bReactionFire = true;              // aim is here. this is causing the camera to disappear
    StandardAim.bAllowCrit = true; 
    StandardAim.bGuaranteedHit = false;    
    StandardAim.bHitsAreCrits = false;      // change this if you want all hits to be crits            
    Template.AbilityToHitCalc = StandardAim;                            // You can mess around above here to change the aim
	 
    Template.AbilityTargetConditions.AddItem(default.LivingHostileUnitDisallowMindControlProperty);
    TargetVisibilityCondition = new class'X2Condition_Visibility';
    TargetVisibilityCondition.bRequireGameplayVisible = true;
    TargetVisibilityCondition.bDisablePeeksOnMovement = true;
    TargetVisibilityCondition.bAllowSquadsight = true;
    Template.AbilityTargetConditions.AddItem(TargetVisibilityCondition);
    AbilityCondition = new class'X2Condition_AbilityProperty';
    AbilityCondition.TargetMustBeInValidTiles = true;
    Template.AbilityTargetConditions.AddItem(AbilityCondition);
    Template.AbilityTargetConditions.AddItem(class'X2Ability_DefaultAbilitySet'.static.OverwatchTargetEffectsCondition());
 
    Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
    Template.AddShooterEffectExclusions();
 
	//  Do not shoot targets that were already hit by this unit this turn with this ability
	KillZoneCondition = new class'X2Condition_UnitEffectsWithAbilitySource';
	KillZoneCondition.AddExcludeEffect('KillZoneTarget', 'AA_UnitIsImmune');
	Template.AbilityTargetConditions.AddItem(KillZoneCondition);
	//  Mark the target as shot by this unit so it cannot be shot again this turn
	KillZoneEffect = new class'X2Effect_Persistent';
	KillZoneEffect.EffectName = 'KillZoneTarget';
	KillZoneEffect.BuildPersistentEffect(1, false, false, false, eGameRule_PlayerTurnBegin);
	KillZoneEffect.SetupEffectOnShotContextResult(true, true);      //  mark them regardless of whether the shot hit or missed
	Template.AddTargetEffect(KillZoneEffect); 
  
    SingleTarget = new class'X2AbilityTarget_Single';
    SingleTarget.OnlyIncludeTargetsInsideWeaponRange = true;
    Template.AbilityTargetStyle = SingleTarget;
 
    //  Put holo target effect first because if the target dies from this shot, it will be too late to notify the effect.
    // holo or shredder here?
    Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.HoloTargetEffect());
    Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.ShredderDamageEffect());
 
    Template.bAllowAmmoEffects = true;
 
    //Trigger on movement - interrupt the move
    Trigger = new class'X2AbilityTrigger_Event';
    Trigger.EventObserverClass = class'X2TacticalGameRuleset_MovementObserver';
    Trigger.MethodName = 'InterruptGameState';
    Template.AbilityTriggers.AddItem(Trigger);
    Trigger = new class'X2AbilityTrigger_Event';
    Trigger.EventObserverClass = class'X2TacticalGameRuleset_AttackObserver';
    Trigger.MethodName = 'InterruptGameState';
    Template.AbilityTriggers.AddItem(Trigger);
 
    Template.AbilitySourceName = 'eAbilitySource_Perk';
    Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
    Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_overwatch";
    Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_MAJOR_PRIORITY;
    Template.bDisplayInUITooltip = false;
    Template.bDisplayInUITacticalText = false;
 
    Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
    Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
 
    return Template;
}
defaultproperties
{
	KillZoneReserveType = "KillZone";
}








//******** Hunter's Instinct 3 Ability **********
//static function X2AbilityTemplate AddHuntersInstinctAbility3()
//{
	//local X2AbilityTemplate						Template;
	//local X2Effect_HuntersInstinctDamage        DamageModifier;
	//local X2Effect_PersistentStatChange     PersistentStatChangeEffect;
//
	//`CREATE_X2ABILITY_TEMPLATE(Template, 'HuntersInstinct3');
//
	//Template.AbilitySourceName = 'eAbilitySource_Perk';
	//Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	//Template.Hostility = eHostility_Neutral;
	//Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_hunter";
//
	//Template.AbilityToHitCalc = default.DeadEye;
	//Template.AbilityTargetStyle = default.SelfTarget;
	//Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);
	//
	//PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	//PersistentStatChangeEffect.AddPersistentStatChange(eStat_ArmorPiercing, 1);
	//PersistentStatChangeEffect.AddPersistentStatChange(eStat_UtilityItems, 2);
	//Template.AddTargetEffect(PersistentStatChangeEffect);
//
	//DamageModifier = new class'X2Effect_HuntersInstinctDamage';
	//DamageModifier.BonusDamage = 1;
	//DamageModifier.BuildPersistentEffect(1, true, true, true);
	//DamageModifier.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, true,,Template.AbilitySourceName);
	//Template.AddTargetEffect(DamageModifier);
//
	//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	////  NOTE: No visualization on purpose!
//
//
	//return Template;
//}

// ********************************DeadEye 2******************************************************8

static function X2AbilityTemplate Deadeye2Ability()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityCooldown                 Cooldown;
	local X2AbilityToHitCalc_StandardAim    ToHitCalc;
	local X2Condition_Visibility            TargetVisibilityCondition;
	local X2AbilityCost_Ammo                AmmoCost;
	local X2AbilityCost_ActionPoints        ActionPointCost;
	local X2Condition_UnitProperty          ShooterCondition;	 //here
	local X2AbilityCharges                      Charges;
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Deadeye2');

	Template.AdditionalAbilities.AddItem('Deadeye2Damage');

	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_precisionshot";
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.Hostility = eHostility_Offensive;
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_COLONEL_PRIORITY;
	Template.AbilityConfirmSound = "TacticalUI_ActivateAbility";

	Template.TargetingMethod = class'X2TargetingMethod_OverTheShoulder';
	Template.bUsesFiringCamera = true;
	Template.CinescriptCameraType = "StandardGunFiring";

	Cooldown = new class'X2AbilityCooldown';
	Cooldown.iNumTurns = 2;
	Template.AbilityCooldown = Cooldown;
	Charges = new class'X2AbilityCharges';
	Charges.InitialCharges = 2; 
	Template.AbilityCharges = Charges;
	
	ToHitCalc = new class'X2AbilityToHitCalc_StandardAim';
	ToHitCalc.FinalMultiplier = 0.25f; //.25f
	Template.AbilityToHitCalc = ToHitCalc;

	AmmoCost = new class'X2AbilityCost_Ammo';
	AmmoCost.iAmmo = 1;
	Template.AbilityCosts.AddItem(AmmoCost);

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

	Template.AbilityTargetStyle = default.SimpleSingleTarget;
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);

	//here
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);	
	ShooterCondition = new class'X2Condition_UnitProperty';
	ShooterCondition.IsUnspotted = true;  //fix this here needs to only be activateable while concealed.
	Template.AbilityShooterConditions.AddItem(ShooterCondition);

	TargetVisibilityCondition = new class'X2Condition_Visibility';
	TargetVisibilityCondition.bRequireGameplayVisible = true;
	TargetVisibilityCondition.bAllowSquadsight = true;
	Template.AbilityTargetConditions.AddItem(TargetVisibilityCondition);
	Template.AbilityTargetConditions.AddItem(default.LivingHostileTargetProperty);

	//  Put holo target effect first because if the target dies from this shot, it will be too late to notify the effect.
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.HoloTargetEffect());
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.ShredderDamageEffect());

	Template.bAllowAmmoEffects = true;

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;

	//here conceal
	Template.ConcealmentRule = eConceal_KillShot;

	Template.bCrossClassEligible = false;

	return Template;
}

static function X2AbilityTemplate Deadeye2Damage()
{
	local X2AbilityTemplate						Template;
	local X2Effect_Deadeye2Damage                DamageEffect;
 
	// Icon Properties
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Deadeye2Damage');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_momentum";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	DamageEffect = new class'X2Effect_Deadeye2Damage';
 	DamageEffect.DamageMultiplier = 0.25;		
	DamageEffect.BuildPersistentEffect(1, true, false, false);
	DamageEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, false,,Template.AbilitySourceName);
	Template.AddTargetEffect(DamageEffect);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//  NOTE: No visualization on purpose!

	return Template;
}


static function X2AbilityTemplate Untouchable2()
{
	local X2AbilityTemplate						Template;
	local X2AbilityTargetStyle                  TargetStyle;
	local X2AbilityTrigger						Trigger;
	local X2Effect_RangerStealth                  UntouchableEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Untouchable2');

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_evasion";

	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;

	Trigger = new class'X2AbilityTrigger_UnitPostBeginPlay';
	Template.AbilityTriggers.AddItem(Trigger);

	//UntouchableEffect = new class'X2Effect_Untouchable';
	//UntouchableEffect.BuildPersistentEffect(1, true, false, false);
//	UntouchableEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
//	UntouchableEffect.DuplicateResponse = eDupe_Ignore;
//	Template.AddTargetEffect(UntouchableEffect);

	//start
	UntouchableEffect = new class'X2Effect_RangerStealth';
	UntouchableEffect.BuildPersistentEffect(1, true, true, false, eGameRule_PlayerTurnEnd);
	UntouchableEffect.SetDisplayInfo(ePerkBuff_Bonus, Template.LocFriendlyName, Template.GetMyHelpText(), Template.IconImage, true);
	UntouchableEffect.bRemoveWhenTargetConcealmentBroken = true;  
	Template.AddTargetEffect(UntouchableEffect);

	Template.AddTargetEffect(class'X2Effect_Spotted'.static.CreateUnspottedEffect());

	Template.ActivationSpeech = 'ActivateConcealment';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
	Template.bSkipFireAction = true;

	//end

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//  NOTE: No visualization on purpose!

	Template.bCrossClassEligible = false;

	return Template;
}

// ********************************KnockoutSelf2*****************************
static function X2DataTemplate AddKnockoutSelfAbility2()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityTrigger_PlayerInput      InputTrigger;
	local X2AbilityCost_ActionPoints        ActionPointCost;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'KnockoutSelf2');

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.AbilitySourceName = 'eAbilitySource_Standard';

	InputTrigger = new class'X2AbilityTrigger_PlayerInput';
	Template.AbilityTriggers.AddItem(InputTrigger);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_ShowIfAvailable;
	Template.Hostility = eHostility_Defensive;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_coupdegrace";
	Template.AddTargetEffect(class'X2StatusEffects'.static.CreateUnconsciousStatusEffect());

	Template.bAllowedByDefault = true;

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState; // just adds the unconscious status effect
	Template.BuildVisualizationFn = Knockout_BuildVisualization;
	 
	return Template;
}

simulated function Knockout_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory History;
	local XComGameStateContext_Ability  Context;
	local StateObjectReference          InteractingUnitRef;
	local XComGameState_Ability         Ability;

	local VisualizationTrack        EmptyTrack;
	local VisualizationTrack        BuildTrack;

	local X2Action_PlaySoundAndFlyOver SoundAndFlyOver;

	History = `XCOMHISTORY;

	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());
	InteractingUnitRef = Context.InputContext.SourceObject;

	//Configure the visualization track for the shooter
	//****************************************************************************************
	BuildTrack = EmptyTrack;
	BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	BuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
	BuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);
					
	Ability = XComGameState_Ability(History.GetGameStateForObjectID(Context.InputContext.AbilityRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1));
	SoundAndFlyOver = X2Action_PlaySoundAndFlyOver(class'X2Action_PlaySoundAndFlyOver'.static.AddToVisualizationTrack(BuildTrack, Context));
	SoundAndFlyOver.SetSoundAndFlyOverParameters(None, Ability.GetMyTemplate().LocFlyOverText, '', eColor_Good);

	class'X2Action_ExitCover'.static.AddToVisualizationTrack(BuildTrack, Context);
	class'X2Action_Knockout'.static.AddToVisualizationTrack(BuildTrack, Context);
	class'X2Action_EnterCover'.static.AddToVisualizationTrack(BuildTrack, Context);
	OutVisualizationTracks.AddItem(BuildTrack);
}


//*************************Stasis Attempt********************************
 
//try changing unconscious to that^

static function X2AbilityTemplate Sustain2()
{
	local X2AbilityTemplate             Template;
	local X2Effect_Sustain              SustainEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Sustain2');

	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_sustain";
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_ShowIfAvailable;
	Template.Hostility = eHostility_Neutral;
	Template.bIsPassive = true;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	SustainEffect = new class'X2Effect_Sustain';
	SustainEffect.BuildPersistentEffect(1, true, true);
	SustainEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, true,, Template.AbilitySourceName);
	Template.AddTargetEffect(SustainEffect);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	// Note: no visualization on purpose!

	//Template.PrerequisiteAbilities.AddItem('Stasis');
	Template.AdditionalAbilities.AddItem('SustainTriggered');

	return Template;
}

static function X2DataTemplate SustainTriggered2()
{
	local X2AbilityTemplate                 Template;
	local X2Effect_Stasis                   StasisEffect;
	local X2AbilityTrigger_EventListener    EventTrigger;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'SustainTriggered2');

	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_sustain";
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
	Template.AbilitySourceName = 'eAbilitySource_Psionic';

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;

	StasisEffect = new class'X2Effect_Stasis';
	StasisEffect.BuildPersistentEffect(2, false, false, false, eGameRule_PlayerTurnBegin);
	StasisEffect.bUseSourcePlayerState = true;
	StasisEffect.bRemoveWhenTargetDies = true;          //  probably shouldn't be possible for them to die while in stasis, but just in case
	StasisEffect.SetDisplayInfo(ePerkBuff_Penalty, Template.LocFriendlyName, Template.GetMyHelpText(), Template.IconImage);
	StasisEffect.StunStartAnim = 'HL_PsiSustainStart'; 
	StasisEffect.CustomIdleOverrideAnim = 'HL_CarryBodyLoop';
	StasisEffect.bSkipFlyover = true;
	Template.AddTargetEffect(StasisEffect);

	EventTrigger = new class'X2AbilityTrigger_EventListener';
	EventTrigger.ListenerData.Deferral = ELD_OnStateSubmitted;
	EventTrigger.ListenerData.EventID = 'SustainTriggered2';
	EventTrigger.ListenerData.Filter = eFilter_Unit;
	EventTrigger.ListenerData.EventFn = class'XComGameState_Ability'.static.AbilityTriggerEventListener_Self_VisualizeInGameState;
	Template.AbilityTriggers.AddItem(EventTrigger);

	Template.PostActivationEvents.AddItem(class'X2Effect_Sustain'.default.SustainTriggeredEvent);
		
	Template.ActivationSpeech = 'Deathscream';
	Template.bSkipFireAction = true;
	//Template.bSkipFireAction = false;
	Template.CinescriptCameraType = "StandardGunFiring";
	//Template.CinescriptCameraType = "Faceless_ChangeForm";
	//Template.CinescriptCameraType = "GenericAccentCam";
	Template.FrameAbilityCameraType = eCameraFraming_Never;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;  //stun smooth
	//Template.BuildVisualizationFn = Knockout_BuildVisualization;  //knockout into stun idle
	//Template.BuildVisualizationFn = Knockout_BuildVisualization;


		//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
////	Template.BuildVisualizationFn = Knockout_BuildVisualization;
	//Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;  ////blinks lay down, then goes to stun idle
////	Template.BuildAffectedVisualizationSyncFn = Knockout_BuildAffectedVisualizationSync;
////	Template.BuildVisualizationFn = StunLancerImpairing_BuildVisualization;  //blinks lay down, then goes to stun idle
 	//Template.CinescriptCameraType = "Psionic_FireAtUnit";
	return Template;
}

//**********************FACEOFF***************************************888

static function X2AbilityTemplate Faceoff2()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityCooldown                 Cooldown;
	local X2AbilityCost_ActionPoints        ActionPointCost;
	local X2AbilityToHitCalc_StandardAim    ToHitCalc;
	local X2AbilityCost_Ammo				AmmoCost;
//	local X2Condition_UnitProperty          PropertyCondition;
 

	`CREATE_X2ABILITY_TEMPLATE(Template, 'Faceoff2');

	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_beserk";
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.Hostility = eHostility_Offensive;
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_COLONEL_PRIORITY;
	Template.AbilityConfirmSound = "TacticalUI_ActivateAbility";

	Cooldown = new class'X2AbilityCooldown';
	Cooldown.iNumTurns = 8;
	Template.AbilityCooldown = Cooldown;

	AmmoCost = new class'X2AbilityCost_Ammo';
	AmmoCost.iAmmo = 1;
	AmmoCost.bFreeCost = false;
	Template.AbilityCosts.AddItem(AmmoCost);

	ToHitCalc = new class'X2AbilityToHitCalc_StandardAim';
	ToHitCalc.bOnlyMultiHitWithSuccess = false;
	ToHitCalc.FinalMultiplier = 0.20f;
	Template.AbilityToHitCalc = ToHitCalc;

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

	Template.AbilityTargetStyle = default.SimpleSingleTarget;
	Template.AbilityMultiTargetStyle = new class'X2AbilityMultiTarget_AllAllies';
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);
	Template.AbilityTargetConditions.AddItem(default.LivingHostileTargetProperty);

	Template.AddTargetEffect(new class'X2Effect_ApplyWeaponDamage');
	Template.AddMultiTargetEffect(new class'X2Effect_ApplyWeaponDamage');
  
	Template.bAllowAmmoEffects = true;
	 
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = Faceoff2_BuildVisualization;
	Template.ActivationSpeech = 'PanickedBreathing';
	Template.AdditionalAbilities.AddItem('SustainTriggered2');
	Template.PostActivationEvents.AddItem('SustainTriggered2');

	return Template;
}

function Faceoff2_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local X2AbilityTemplate             AbilityTemplate;
	local XComGameStateContext_Ability  Context;
	local AbilityInputContext           AbilityContext;
	local StateObjectReference          ShootingUnitRef;	
	local X2Action_Fire                 FireAction;
	local X2Action_Fire_Faceoff         FireFaceoffAction;
	local XComGameState_BaseObject      TargetStateObject;//Container for state objects within VisualizeGameState	
	
	local Actor                     TargetVisualizer, ShooterVisualizer;
	local X2VisualizerInterface     TargetVisualizerInterface;
	local int                       EffectIndex, TargetIndex;

	local VisualizationTrack        EmptyTrack;
	local VisualizationTrack        BuildTrack;
	local VisualizationTrack        SourceTrack;
	local XComGameStateHistory      History;

	local X2Action_PlaySoundAndFlyOver SoundAndFlyover;
	local name         ApplyResult;

	local X2Action_StartCinescriptCamera CinescriptStartAction;
	local X2Action_EndCinescriptCamera   CinescriptEndAction;
	local X2Camera_Cinescript            CinescriptCamera;
	local string                         PreviousCinescriptCameraType;
	local X2Effect                       TargetEffect;


	History = `XCOMHISTORY;
	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());
	AbilityContext = Context.InputContext;
	AbilityTemplate = class'XComGameState_Ability'.static.GetMyTemplateManager().FindAbilityTemplate(AbilityContext.AbilityTemplateName);
	ShootingUnitRef = Context.InputContext.SourceObject;

	ShooterVisualizer = History.GetVisualizer(ShootingUnitRef.ObjectID);

	SourceTrack = EmptyTrack;
	SourceTrack.StateObject_OldState = History.GetGameStateForObjectID(ShootingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	SourceTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(ShootingUnitRef.ObjectID);
	if (SourceTrack.StateObject_NewState == none)
		SourceTrack.StateObject_NewState = SourceTrack.StateObject_OldState;
	SourceTrack.TrackActor = ShooterVisualizer;

	if (AbilityTemplate.ActivationSpeech != '')     //  allows us to change the template without modifying this function later
	{
		SoundAndFlyOver = X2Action_PlaySoundAndFlyOver(class'X2Action_PlaySoundAndFlyover'.static.AddToVisualizationTrack(SourceTrack, Context));
		SoundAndFlyOver.SetSoundAndFlyOverParameters(None, "", AbilityTemplate.ActivationSpeech, eColor_Good);
	}


	// Add a Camera Action to the Shooter's track.  Minor hack: To create a CinescriptCamera the AbilityTemplate 
	// must have a camera type.  So manually set one here, use it, then restore.
	PreviousCinescriptCameraType = AbilityTemplate.CinescriptCameraType;
	AbilityTemplate.CinescriptCameraType = "StandardGunFiring";
	CinescriptCamera = class'X2Camera_Cinescript'.static.CreateCinescriptCameraForAbility(Context);
	CinescriptStartAction = X2Action_StartCinescriptCamera( class'X2Action_StartCinescriptCamera'.static.AddToVisualizationTrack(SourceTrack, Context ) );
	CinescriptStartAction.CinescriptCamera = CinescriptCamera;
	AbilityTemplate.CinescriptCameraType = PreviousCinescriptCameraType;


	class'X2Action_ExitCover'.static.AddToVisualizationTrack(SourceTrack, Context);
	
	//  Fire at the primary target first
	FireAction = X2Action_Fire(class'X2Action_Fire'.static.AddToVisualizationTrack(SourceTrack, Context));
	FireAction.SetFireParameters(Context.IsResultContextHit(), , false);
	//  Setup target response
	TargetVisualizer = History.GetVisualizer(AbilityContext.PrimaryTarget.ObjectID);
	TargetVisualizerInterface = X2VisualizerInterface(TargetVisualizer);
	BuildTrack = EmptyTrack;
	BuildTrack.TrackActor = TargetVisualizer;
	TargetStateObject = VisualizeGameState.GetGameStateForObjectID(AbilityContext.PrimaryTarget.ObjectID);
	if( TargetStateObject != none )
	{
		History.GetCurrentAndPreviousGameStatesForObjectID(AbilityContext.PrimaryTarget.ObjectID, 
															BuildTrack.StateObject_OldState, BuildTrack.StateObject_NewState,
															eReturnType_Reference,
															VisualizeGameState.HistoryIndex);
		`assert(BuildTrack.StateObject_NewState == TargetStateObject);
	}
	else
	{
		//If TargetStateObject is none, it means that the visualize game state does not contain an entry for the primary target. Use the history version
		//and show no change.
		BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(AbilityContext.PrimaryTarget.ObjectID);
		BuildTrack.StateObject_NewState = BuildTrack.StateObject_OldState;
	}
	class'X2Action_WaitForAbilityEffect'.static.AddToVisualizationTrack(BuildTrack, Context);
	for (EffectIndex = 0; EffectIndex < AbilityTemplate.AbilityTargetEffects.Length; ++EffectIndex)
	{
		ApplyResult = Context.FindTargetEffectApplyResult(AbilityTemplate.AbilityTargetEffects[EffectIndex]);

		// Target effect visualization
		AbilityTemplate.AbilityTargetEffects[EffectIndex].AddX2ActionsForVisualization(VisualizeGameState, BuildTrack, ApplyResult);

		// Source effect visualization
		AbilityTemplate.AbilityTargetEffects[EffectIndex].AddX2ActionsForVisualizationSource(VisualizeGameState, SourceTrack, ApplyResult);
	}
	if( TargetVisualizerInterface != none )
	{
		//Allow the visualizer to do any custom processing based on the new game state. For example, units will create a death action when they reach 0 HP.
		TargetVisualizerInterface.BuildAbilityEffectsVisualization(VisualizeGameState, BuildTrack);
	}
	OutVisualizationTracks.AddItem(BuildTrack);
	
	//  Now configure a fire action for each multi target
	for (TargetIndex = 0; TargetIndex < AbilityContext.MultiTargets.Length; ++TargetIndex)
	{
		// Add an action to pop the previous CinescriptCamera off the camera stack.
		CinescriptEndAction = X2Action_EndCinescriptCamera( class'X2Action_EndCinescriptCamera'.static.AddToVisualizationTrack( SourceTrack, Context ) );
		CinescriptEndAction.CinescriptCamera = CinescriptCamera;
		CinescriptEndAction.bForceEndImmediately = true;

		// Add an action to push a new CinescriptCamera onto the camera stack.
		AbilityTemplate.CinescriptCameraType = "StandardGunFiring";
		CinescriptCamera = class'X2Camera_Cinescript'.static.CreateCinescriptCameraForAbility(Context);
		CinescriptCamera.TargetObjectIdOverride = AbilityContext.MultiTargets[TargetIndex].ObjectID;
		CinescriptStartAction = X2Action_StartCinescriptCamera( class'X2Action_StartCinescriptCamera'.static.AddToVisualizationTrack(SourceTrack, Context ) );
		CinescriptStartAction.CinescriptCamera = CinescriptCamera;
		AbilityTemplate.CinescriptCameraType = PreviousCinescriptCameraType;

		// Add a custom Fire action to the shooter track.
		TargetVisualizer = History.GetVisualizer(AbilityContext.MultiTargets[TargetIndex].ObjectID);
		FireFaceoffAction = X2Action_Fire_Faceoff(class'X2Action_Fire_Faceoff'.static.AddToVisualizationTrack(SourceTrack, Context));
		FireFaceoffAction.SetFireParameters(Context.IsResultContextMultiHit(TargetIndex), AbilityContext.MultiTargets[TargetIndex].ObjectID, false);
		FireFaceoffAction.vTargetLocation = TargetVisualizer.Location;


		//  Setup target response
		TargetVisualizerInterface = X2VisualizerInterface(TargetVisualizer);
		BuildTrack = EmptyTrack;
		BuildTrack.TrackActor = TargetVisualizer;
		TargetStateObject = VisualizeGameState.GetGameStateForObjectID(AbilityContext.MultiTargets[TargetIndex].ObjectID);
		if( TargetStateObject != none )
		{
			History.GetCurrentAndPreviousGameStatesForObjectID(AbilityContext.MultiTargets[TargetIndex].ObjectID, 
																BuildTrack.StateObject_OldState, BuildTrack.StateObject_NewState,
																eReturnType_Reference,
																VisualizeGameState.HistoryIndex);
			`assert(BuildTrack.StateObject_NewState == TargetStateObject);
		}
		else
		{
			//If TargetStateObject is none, it means that the visualize game state does not contain an entry for the primary target. Use the history version
			//and show no change.
			BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(AbilityContext.MultiTargets[TargetIndex].ObjectID);
			BuildTrack.StateObject_NewState = BuildTrack.StateObject_OldState;
		}

		// Add WaitForAbilityEffect. To avoid time-outs when there are many targets, set a custom timeout
		class'X2Action_WaitForAbilityEffect'.static.AddToVisualizationTrack(BuildTrack, Context);
		BuildTrack.TrackActions[BuildTrack.TrackActions.Length - 1].SetCustomTimeoutSeconds((10 + (10 * TargetIndex )));

		for (EffectIndex = 0; EffectIndex < AbilityTemplate.AbilityMultiTargetEffects.Length; ++EffectIndex)
		{
			TargetEffect = AbilityTemplate.AbilityMultiTargetEffects[EffectIndex];
			ApplyResult = Context.FindMultiTargetEffectApplyResult(TargetEffect, TargetIndex);

			// Target effect visualization
			AbilityTemplate.AbilityMultiTargetEffects[EffectIndex].AddX2ActionsForVisualization(VisualizeGameState, BuildTrack, ApplyResult);

			// If the last Effect applied was weapon damage, then a weapon damage Action was added to the track.
			// Find that weapon damage action, and extend its timeout so that we won't timeout if there are many
			// targets to visualize before this one.
			if ( X2Effect_ApplyWeaponDamage(TargetEffect) != none )
			{
				if ( X2Action_ApplyWeaponDamageToUnit(BuildTrack.TrackActions[BuildTrack.TrackActions.Length - 1]) != none)
				{
					BuildTrack.TrackActions[BuildTrack.TrackActions.Length - 1].SetCustomTimeoutSeconds((10 + (10 * TargetIndex )));
				}
			}

			// Source effect visualization
			AbilityTemplate.AbilityMultiTargetEffects[EffectIndex].AddX2ActionsForVisualizationSource(VisualizeGameState, SourceTrack, ApplyResult);
		}
		if( TargetVisualizerInterface != none )
		{
			//Allow the visualizer to do any custom processing based on the new game state. For example, units will create a death action when they reach 0 HP.
			TargetVisualizerInterface.BuildAbilityEffectsVisualization(VisualizeGameState, BuildTrack);
		}
		OutVisualizationTracks.AddItem(BuildTrack);
	}
	class'X2Action_EnterCover'.static.AddToVisualizationTrack(SourceTrack, Context);

	// Add an action to pop the last CinescriptCamera off the camera stack.
	CinescriptEndAction = X2Action_EndCinescriptCamera( class'X2Action_EndCinescriptCamera'.static.AddToVisualizationTrack( SourceTrack, Context ) );
	CinescriptEndAction.CinescriptCamera = CinescriptCamera;

	OutVisualizationTracks.AddItem(SourceTrack);



}

static function X2Effect_Persistent SharpshooterAimEffect()
{
	local X2Effect_SharpshooterAim      AimEffect;
	local X2Condition_AbilityProperty   AbilityCondition;

	AimEffect = new class'X2Effect_SharpshooterAim';
	AimEffect.BuildPersistentEffect(2, false, true, false, eGameRule_PlayerTurnEnd);
	AimEffect.SetDisplayInfo(ePerkBuff_Bonus, default.SharpshooterAimBonusName, default.SharpshooterAimBonusDesc, "img:///UILibrary_PerkIcons.UIPerk_aim");
	
	AbilityCondition = new class'X2Condition_AbilityProperty';
	AbilityCondition.OwnerHasSoldierAbilities.AddItem('SharpshooterAim');
	AimEffect.TargetConditions.AddItem(AbilityCondition);

	return AimEffect;
}




static function X2DataTemplate Stasis2()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityCost_ActionPoints        ActionPointCost;
	local X2Effect_Stasis2                   StasisEffect;
//	local X2AbilityCooldown                 Cooldown;
	local X2AbilityTrigger_EventListener    Trigger;
	
	`CREATE_X2ABILITY_TEMPLATE(Template, 'Stasis2');

	Template.Hostility = eHostility_Offensive;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_stasis";
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_ShowIfAvailable;  //showifavailable
	//Template.AbilitySourceName = 'eAbilitySource_Psionic';
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	
	Trigger = new class'X2AbilityTrigger_EventListener';
	Trigger.ListenerData.Deferral = ELD_OnStateSubmitted;
	Trigger.ListenerData.EventID = 'Stasis2';
	Trigger.ListenerData.Filter = eFilter_Unit;
 	//Trigger.ListenerData.EventFn = class'XComGameState_Ability'.static.RapidFireListener;  //fixthisthere
	Trigger.ListenerData.EventFn = class'XComGameState_Ability'.static.AbilityTriggerEventListener_Self_VisualizeInGameState;
	Template.AbilityTriggers.AddItem(Trigger);
	
	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 0;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_SQUADDIE_PRIORITY;

//	Cooldown = new class'X2AbilityCooldown';
//	Cooldown.iNumTurns = 2;
//	Template.AbilityCooldown = Cooldown;

	Template.AbilityToHitCalc = default.DeadEye;

//	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
//	Template.AddShooterEffectExclusions();  //here?

	//Template.AbilityTargetConditions.AddItem(new class'X2Condition_StasisTarget');  //here?
//	Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);

	StasisEffect = new class'X2Effect_Stasis2';
	StasisEffect.BuildPersistentEffect(2, false, false, false, eGameRule_PlayerTurnBegin);
	StasisEffect.bUseSourcePlayerState = true;
	StasisEffect.bRemoveWhenTargetDies = true;          //  probably shouldn't be possible for them to die while in stasis, but just in case
	StasisEffect.SetDisplayInfo(ePerkBuff_Penalty, Template.LocFriendlyName, Template.GetMyHelpText(), Template.IconImage);
	Template.AddTargetEffect(StasisEffect);

//	Template.AbilityTargetStyle = default.SingleTargetWithSelf;
//	Template.AbilityTargetStyle = default.SelfTarget;
//	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);
	Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');
	//Template.bShowActivation = true;
	//Template.CustomFireAnim = 'HL_Psi_SelfCast';
	//Template.ActivationSpeech = 'NullShield';

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = Knockout_BuildVisualization;
 	Template.CinescriptCameraType = "Psionic_FireAtUnit";
	Template.bShowActivation = true;

	return Template;
}

//**************TEMP RAPID FIRE **********

static function X2AbilityTemplate RapidFire3()
{
	local X2AbilityTemplate					Template;
	local X2AbilityCost_ActionPoints		ActionPointCost;
	local X2AbilityCost_Ammo				AmmoCost;
	local X2AbilityToHitCalc_StandardAim    ToHitCalc;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'RapidFire3');

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 0;
	ActionPointCost.bAddWeaponTypicalCost = true;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

	//  require 2 ammo to be present so that both shots can be taken
	AmmoCost = new class'X2AbilityCost_Ammo';
	AmmoCost.iAmmo = 2;
	AmmoCost.bFreeCost = true;
	Template.AbilityCosts.AddItem(AmmoCost);
	//  actually charge 1 ammo for this shot. the 2nd shot will charge the extra ammo.
	AmmoCost = new class'X2AbilityCost_Ammo';
	AmmoCost.iAmmo = 1;
	Template.AbilityCosts.AddItem(AmmoCost);

	ToHitCalc = new class'X2AbilityToHitCalc_StandardAim';
	ToHitCalc.BuiltInHitMod = -15;
	Template.AbilityToHitCalc = ToHitCalc;
	Template.AbilityToHitOwnerOnMissCalc = ToHitCalc;

	Template.AbilityTargetStyle = default.SimpleSingleTarget;

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	Template.AbilityTargetConditions.AddItem(default.LivingHostileTargetProperty);
	Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);

	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.HoloTargetEffect());
	Template.AssociatedPassives.AddItem('HoloTargeting');
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.ShredderDamageEffect());
	Template.bAllowAmmoEffects = true;

	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);

	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_COLONEL_PRIORITY;
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_rapidfire";
	Template.AbilityConfirmSound = "TacticalUI_ActivateAbility";
	Template.CinescriptCameraType = "StandardGunFiring";
	Template.TargetingMethod = class'X2TargetingMethod_OverTheShoulder';

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
	Template.BuildInterruptGameStateFn = TypicalAbility_BuildInterruptGameState;

	Template.AdditionalAbilities.AddItem('Stasis2');
	Template.PostActivationEvents.AddItem('Stasis2');

	Template.bCrossClassEligible = false;
	Template.bPreventsTargetTeleport = true;

	return Template;
}

//*********ADD LOW PROFILE**********

//******** kernEye Ability (GTS) **********
static function X2AbilityTemplate IsmsLowProfile()
{
	local X2AbilityTemplate						Template;
	local X2AbilityTargetStyle                  TargetStyle;
	local X2AbilityTrigger						Trigger;
//	local X2Effect_ToHitModifier                ToHitModifier;
	local X2Effect_LowProfile					LowProfileEffect;
//	local X2Effect_HuntersInstinctDamage        DamageModifier;
	local X2Effect_PersistentStatChange     PersistentStatChangeEffect;
	local X2Condition_Visibility				TargetVisibilityCondition;
	
	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsLowProfile');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_LowProfile";
	//dd
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;

 

//	TargetStyle = new class'X2AbilityTarget_Self';
//	Template.AbilityTargetStyle = TargetStyle;

//	Trigger = new class'X2AbilityTrigger_UnitPostBeginPlay';
//	Template.AbilityTriggers.AddItem(Trigger);

	//try this AbilityShooterConditions
	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
 	Template.bIsPassive = true;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	//X2GameRulesetVisibilityDataStructures 

	LowProfileEffect = new class'X2Effect_LowProfile';
	LowProfileEffect.BuildPersistentEffect(1, true, false, false);
	LowProfileEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, true,,Template.AbilitySourceName);
	Template.AddTargetEffect(LowProfileEffect);

////	TargetVisibilityCondition = new class'X2Condition_Visibility';
	//TargetVisibilityCondition.bRequireGameplayVisible = true;
	//TargetVisibilityCondition.bCannotPeek = true;  
	//TargetVisibilityCondition.bRequireNotMatchCoverType = true;
////	TargetVisibilityCondition.bRequireMatchCoverType = true;
////	TargetVisibilityCondition.TargetCover = CT_Midlevel;   //CT_Standing
////	Template.AbilityShooterConditions.AddItem(TargetVisibilityCondition);

	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
//	PersistentStatChangeEffect.AddPersistentStatChange(eStat_FlankingCritChance, 4);
//	PersistentStatChangeEffect.AddPersistentStatChange(eStat_FlankingAimBonus, 5);
//	PersistentStatChangeEffect.AddPersistentStatChange(eStat_ArmorPiercing, 1);
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Offense, -5);
	//PersistentStatChangeEffect.AddPersistentStatChange(eStat_DetectionModifier, .2f);
	Template.AddTargetEffect(PersistentStatChangeEffect);
	

	
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	
	//  NOTE: No visualization on purpose!

	return Template;
}


/////////////////////////Spotter Skill****************************************************


static function X2AbilityTemplate RogueSpotter()
{
	local X2AbilityTemplate					Template;
	local X2AbilityTarget_Cursor			CursorTarget;
//	local X2AbilityMultiTarget_Line         LineMultiTarget;
	local X2Condition_UnitProperty          TargetCondition;
	local X2AbilityCost_ActionPoints        ActionCost;
 	local X2AbilityCooldown_PerPlayerType	Cooldown;
	local X2Effect_Persistent				MarkedEffect; 
	local X2AbilityMultiTarget_Cone     ConeMultiTarget;
	local X2Condition_UnitProperty			UnitPropertyCondition;
//	local X2Condition_UnitEffects			UnitEffectsCondition;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'RogueSpotter');

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;
	//Template.Hostility = eHostility_Offensive;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_fire_control";
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_SQUADDIE_PRIORITY;

 
	Template.CustomFireAnim = 'HL_SignalPoint';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;		
 	Template.BuildVisualizationFn = class'X2Ability_AdventCaptain'.static.TargetGettingMarked_BuildVisualization;
	Template.BuildInterruptGameStateFn = TypicalAbility_BuildInterruptGameState;
	Template.CinescriptCameraType = "Mark_Target";
	
	ActionCost = new class'X2AbilityCost_ActionPoints';
	ActionCost.iNumPoints = 1;    
	ActionCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionCost);

	Cooldown = new class'X2AbilityCooldown_PerPlayerType';
	Cooldown.iNumTurns = 3;
	Cooldown.iNumTurnsForAI = 3;
	cooldown.NumGlobalTurns = 3;
	Template.AbilityCooldown = Cooldown;

	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);	
	Template.AbilityToHitCalc = default.DeadEye;

	CursorTarget = new class'X2AbilityTarget_Cursor';
	CursorTarget.FixedAbilityRange = 15;
	Template.AbilityTargetStyle = CursorTarget;

//	LineMultiTarget = new class'X2AbilityMultiTarget_Line';
//	Template.AbilityMultiTargetStyle = LineMultiTarget;

	ConeMultiTarget = new class'X2AbilityMultiTarget_Cone';
	ConeMultiTarget.bUseWeaponRadius = true;
	ConeMultiTarget.ConeEndDiameter = 6.0* class'XComWorldData'.const.WORLD_StepSize;
	ConeMultiTarget.ConeLength = 19.0 * class'XComWorldData'.const.WORLD_StepSize;
	Template.AbilityMultiTargetStyle = ConeMultiTarget;

	UnitPropertyCondition = new class'X2Condition_UnitProperty';
	UnitPropertyCondition.ExcludeDead = true;
	UnitPropertyCondition.ExcludeFriendlyToSource = true;
	UnitPropertyCondition.ExcludeHostileToSource = false;
	UnitPropertyCondition.ExcludeCivilian = true;
	UnitPropertyCondition.FailOnNonUnits = true;
	Template.AbilityMultiTargetConditions.AddItem(UnitPropertyCondition);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	TargetCondition = new class'X2Condition_UnitProperty';
	TargetCondition.ExcludeFriendlyToSource = true;
	TargetCondition.ExcludeDead = true;
	Template.AbilityMultiTargetConditions.AddItem(TargetCondition);
 
 	// Target cannot already be marked
//	UnitEffectsCondition = new class'X2Condition_UnitEffects';
//	UnitEffectsCondition.AddExcludeEffect(class'X2StatusEffects'.default.MarkedName, 'AA_UnitIsMarked');
//	Template.AbilityTargetConditions.AddItem(UnitEffectsCondition);

		//// Create the Marked effect
	MarkedEffect = class'X2Effect_RogueCreateMarkedEffect'.static.CreateMarkedEffect(2, false);
	Template.AddTargetEffect(MarkedEffect); //BMU - changing to an immediate execution for evaluation
	
 
	Template.AddMultiTargetEffect(MarkedEffect);

	//Template.TargetingMethod = class'X2TargetingMethod_Line';
	Template.TargetingMethod = class'X2TargetingMethod_Cone';
					 
//	Template.CinescriptCameraType = "Psionic_FireAtLocation";


	Template.ActivationSpeech = 'TargetSpottedHidden';

	Template.bOverrideAim = true;
	Template.bUseSourceLocationZToAim = true;
	Template.ConcealmentRule = eConceal_Always;
	//do i keep this here? Test

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildInterruptGameStateFn = TypicalAbility_BuildInterruptGameState;
//	Template.BuildVisualizationFn = TargetGettingMarked_BuildVisualization;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
 	Template.CinescriptCameraType = "Mark_Target";	
	return Template;
}


static function X2AbilityTemplate RogueStealthBonus()
{
	local X2AbilityTemplate						Template;
	local X2Effect_PersistentStatChange			PersistentEffect;
	local X2Effect_ToHitModifier                Effect;
	local X2Condition_Visibility                VisCondition;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'RogueStealthBonus');
//	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_stealth"; //shadowstep?
	Template.IconImage = "img:///UILibrary_Rogue2.UIPerk_stealth_blaze";

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.bIsPassive = true;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);

	PersistentEffect = new class'X2Effect_PersistentStatChange';
	PersistentEffect.BuildPersistentEffect(1, true, false);
	PersistentEffect.AddPersistentStatChange(eStat_DetectionModifier, 0.35f);
	PersistentEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	Template.AddTargetEffect(PersistentEffect);

	Effect = new class'X2Effect_ToHitModifier';
	Effect.EffectName = 'Shadowborn';   //here here
	Effect.DuplicateResponse = eDupe_Ignore;
	Effect.BuildPersistentEffect(1, true, false);
//	Effect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	Effect.AddEffectHitModifier(eHit_Success, 15, Template.LocFriendlyName);
	Effect.AddEffectHitModifier(eHit_Crit, 15, Template.LocFriendlyName);
	VisCondition = new class'X2Condition_Visibility';
	VisCondition.bExcludeGameplayVisible = true;
	Effect.ToHitConditions.AddItem(VisCondition);
	Template.AddTargetEffect(Effect);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;

	return Template;
}

// Riposte Below  bIsASuppressionEffect will make the visualization a miss even if it's a hit!
static function X2AbilityTemplate ReturnFire3()
{
	local X2AbilityTemplate						Template;
	local X2AbilityTargetStyle                  TargetStyle;
	local X2AbilityTrigger						Trigger;
	local X2Effect_ReturnFire                   FireEffect;
	local X2AbilityCost_ActionPoints        ActionPointCost; 
	local X2AbilityCooldown                 Cooldown;
	local X2Effect_Persistent				MarkedEffect; 
	local X2Condition_UnitProperty          ShooterPropertyCondition;
	//local X2Effect_Marked                   MarkedEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'ReturnFire3');
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_Riposte";

	Template.AdditionalAbilities.AddItem('ConcussiveShot2');
	Template.AdditionalAbilities.AddItem('RiposteShot');
//	Template.AdditionalAbilities.AddItem('RiposteShot6');

//	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_ShowIfAvailable;
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_LIEUTENANT_PRIORITY;
	//Sound Test Here
	//Template.AbilityConfirmSound = "SoundCivilianFemaleScreams.CivFemaleScreamsOffscreenCue";
	//Sound test here
	
	Template.Hostility = eHostility_Neutral;

	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;

	//Trigger = new class'X2AbilityTrigger_UnitPostBeginPlay';
	Trigger = new class'X2AbilityTrigger_PlayerInput';
	Template.AbilityTriggers.AddItem(Trigger);

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

 	Cooldown = new class'X2AbilityCooldown';
	Cooldown.iNumTurns = 2;
	Template.AbilityCooldown = Cooldown;

	ShooterPropertyCondition = new class'X2Condition_UnitProperty';	
	ShooterPropertyCondition.ExcludeDead = true;                    //Can't shoot while dead
	//ShooterPropertyCondition.ExcludePanicked = true;
	ShooterPropertyCondition.ExcludeInStasis = true;
	ShooterPropertyCondition.ExcludeStunned = true;
	Template.AbilityShooterConditions.AddItem(ShooterPropertyCondition);

	FireEffect = new class'X2Effect_ReturnFire';
	FireEffect.AbilityToActivate = 'ConcussiveShot2';  //here here here here here below false to true
	FireEffect.bPreEmptiveFire = true;
	FireEffect.MaxPointsPerTurn = 1;
	FireEffect.GrantActionPoint = 'riposte';
	FireEffect.BuildPersistentEffect(1, false, false, false, eGameRule_PlayerTurnBegin);
	FireEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	Template.AddTargetEffect(FireEffect);
	
	MarkedEffect = class'X2Effect_RogueCreateMarkedEffect'.static.CreateTauntEffect(2, false);
 
	Template.AddTargetEffect(MarkedEffect); 
 
  


	//Template.ActivationSpeech = 'SoldierRestsMindControl';

	Template.CustomFireAnim = 'HL_Psi_SelfCast';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
//	Template.BuildVisualizationFn = None;
	Template.BuildInterruptGameStateFn = TypicalAbility_BuildInterruptGameState;
	//Template.BuildVisualizationFn = TargetGettingMarked_BuildVisualization;
	Template.BuildVisualizationFn = Riposte_BuildVisualization;
	//  NOTE: No visualization on purpose!

	Template.bCrossClassEligible = false;       //  this can only work with pistols, which only sharpshooters have

	return Template;
}


simulated function Riposte_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory History;
	local XComGameStateContext_Ability  Context;
	local StateObjectReference          InteractingUnitRef;
	local X2AbilityTemplate             AbilityTemplate;
	local XComGameState_Unit            UnitState;

	local VisualizationTrack        EmptyTrack;
	local VisualizationTrack        BuildTrack;

	local X2Action_PlaySoundAndFlyOver SoundAndFlyOver;

	History = `XCOMHISTORY;

	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());
	InteractingUnitRef = Context.InputContext.SourceObject;

	//Configure the visualization track for the shooter
	//****************************************************************************************	
	BuildTrack = EmptyTrack;
	BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	BuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
	BuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

	UnitState = XComGameState_Unit(BuildTrack.StateObject_NewState);
	
	//Civilians on the neutral team are not allowed to have sound + flyover for hunker down
	if( UnitState.GetTeam() != eTeam_Neutral )
	{
		
		SoundAndFlyOver = X2Action_PlaySoundAndFlyOver(class'X2Action_PlaySoundAndFlyOver'.static.AddToVisualizationTrack(BuildTrack, Context));
		if (UnitState.HasSoldierAbility('SharpshooterAim'))
		{
			AbilityTemplate = class'X2AbilityTemplateManager'.static.GetAbilityTemplateManager().FindAbilityTemplate('SharpshooterAim');
		}
		else if (UnitState.HasSoldierAbility('DeepCover') && XComGameState_Unit(BuildTrack.StateObject_OldState).ActionPoints.Find(class'X2CharacterTemplateManager'.default.DeepCoverActionPoint) != INDEX_NONE)
		{
			AbilityTemplate = class'X2AbilityTemplateManager'.static.GetAbilityTemplateManager().FindAbilityTemplate('DeepCover');
		}
		else
		{
			AbilityTemplate = class'X2AbilityTemplateManager'.static.GetAbilityTemplateManager().FindAbilityTemplate(Context.InputContext.AbilityTemplateName);
		}
		SoundAndFlyOver.SetSoundAndFlyOverParameters(SoundCue'SoundUI.HunkerDownCue', AbilityTemplate.LocFlyOverText, 'HunkerDown', eColor_Good, AbilityTemplate.IconImage);
		OutVisualizationTracks.AddItem(BuildTrack);
	}
	//****************************************************************************************
}


static function X2AbilityTemplate ConcussiveShot2Ability()
{
	local X2AbilityTemplate                 Template;
  
	local X2Condition_Visibility            TargetVisibilityCondition;
 
 	local X2AbilityToHitCalc_StandardAim    ToHitCalc;
	local X2Condition_UnitProperty          TargetPropertyCondition;
	local X2Condition_UnitProperty          ShooterPropertyCondition;
	local X2Effect_ImmediateAbilityActivation ImpairingAbilityEffect;
	//local X2Effect_ImmediateAbilityActivation ImpairingAbilityEffect2;
	//local X2Effect_ApplyWeaponDamage        WeaponDamageEffect;
	local X2AbilityCost_ReserveActionPoints ReserveActionPointCost;
	local X2AbilityCooldown                 Cooldown;
	local X2Effect_RemoveEffects			RemoveEffects;
	//local X2Effect_Persistent				DisorientedEffect;
	local X2Effect_Persistent				DisorientedEffect;
 
	`CREATE_X2ABILITY_TEMPLATE(Template, 'ConcussiveShot2');

// 	Template.AdditionalAbilities.AddItem('RiposteShot');
//	Template.AdditionalAbilities.AddItem('ReturnFire3');

	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_Riposte";
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_ShowIfAvailable;
	Template.Hostility = eHostility_Offensive;
	Template.ShotHUDPriority = class'UIUtilities_Tactical'.const.CLASS_LIEUTENANT_PRIORITY;
	Template.AbilityConfirmSound = "TacticalUI_ActivateAbility";

	Template.TargetingMethod = class'X2TargetingMethod_OverTheShoulder';
	Template.bUsesFiringCamera = true;
	Template.CinescriptCameraType = "StandardGunFiring";

 
	ToHitCalc = new class'X2AbilityToHitCalc_StandardAim';
//	ToHitCalc.FinalMultiplier = -0.20f;
	//ToHitCalc.bGuaranteedHit = true;
	ToHitCalc.bAllowCrit = true;
	ToHitCalc.BuiltInHitMod = 20;
	Template.AbilityToHitCalc = ToHitCalc;
 
	Cooldown = new class'X2AbilityCooldown';
	Cooldown.iNumTurns = 2;
	Template.AbilityCooldown = Cooldown;


 	Template.bDontDisplayInAbilitySummary = true;
 	ReserveActionPointCost = new class'X2AbilityCost_ReserveActionPoints';
 	ReserveActionPointCost.iNumPoints = 1;
 	ReserveActionPointCost.AllowedTypes.AddItem('riposte'); //here here
 	Template.AbilityCosts.AddItem(ReserveActionPointCost);
 
 	RemoveEffects = new class'X2Effect_RemoveEffects';
 	RemoveEffects.EffectNamesToRemove.AddItem('Taunt');
 	Template.AddShooterEffect(RemoveEffects);
 

	Template.AbilityTargetStyle = default.SimpleSingleTarget;
//	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AddShooterEffectExclusions();

	TargetVisibilityCondition = new class'X2Condition_Visibility';
	TargetVisibilityCondition.bRequireGameplayVisible = true;
	TargetVisibilityCondition.bAllowSquadsight = true;
	Template.AbilityTargetConditions.AddItem(TargetVisibilityCondition);
	Template.AbilityTargetConditions.AddItem(default.LivingHostileTargetProperty);

	//  Put holo target effect first because if the target dies from this shot, it will be too late to notify the effect.
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.HoloTargetEffect());
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.ShredderDamageEffect());

	Template.bAllowAmmoEffects = true;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;

	Template.bCrossClassEligible = false;

	// Target Conditions
	//
	TargetPropertyCondition = new class'X2Condition_UnitProperty';	
	TargetPropertyCondition.ExcludeDead = true;                     //Can't target dead
	TargetPropertyCondition.ExcludeFriendlyToSource = true;         //Can't target friendlies
	Template.AbilityTargetConditions.AddItem(TargetPropertyCondition);

	// Shooter Conditions
	//
	ShooterPropertyCondition = new class'X2Condition_UnitProperty';	
	ShooterPropertyCondition.ExcludeDead = true;                    //Can't shoot while dead
	Template.AbilityShooterConditions.AddItem(ShooterPropertyCondition);

	//Impairing effects need to come before the damage. This is needed for proper visualization ordering.
	//Effect on a successful melee attack is triggering the Apply Impairing Effect Ability
	ImpairingAbilityEffect = new class 'X2Effect_ImmediateAbilityActivation';
	ImpairingAbilityEffect.BuildPersistentEffect(1, false, true, , eGameRule_PlayerTurnBegin);
	ImpairingAbilityEffect.EffectName = 'ImmediateStunImpair';
	ImpairingAbilityEffect.AbilityName = 'RiposteShot';
	ImpairingAbilityEffect.bRemoveWhenTargetDies = true;
	ImpairingAbilityEffect.VisualizationFn = ImpairingAbilityEffectTriggeredVisualization;
	Template.AddTargetEffect(ImpairingAbilityEffect);

//	DisorientedEffect = class'X2StatusEffects'.static.CreateDisorientedStatusEffect();
	DisorientedEffect = class'X2Effect_RogueCreateMarkedEffect'.static.CreateParriedEffect(1, false); 
	DisorientedEffect.BuildPersistentEffect(1,, false,,eGameRule_PlayerTurnBegin);
	DisorientedEffect.bRemoveWhenSourceDies = false;
	Template.AddTargetEffect(DisorientedEffect);

//	ImpairingAbilityEffect2 = new class 'X2Effect_ImmediateAbilityActivation';
//	ImpairingAbilityEffect2.BuildPersistentEffect(1, false, true, , eGameRule_PlayerTurnBegin);
//	ImpairingAbilityEffect2.EffectName = 'ImmediateStunImpair';
//	ImpairingAbilityEffect2.AbilityName = 'RiposteShot';
//	ImpairingAbilityEffect2.bRemoveWhenTargetDies = true;
//	ImpairingAbilityEffect2.VisualizationFn = ImpairingAbilityEffectTriggeredVisualization;
//	Template.AddTargetEffect(ImpairingAbilityEffect2);

	// Damage Effect
	//
	//WeaponDamageEffect = new class'X2Effect_ApplyWeaponDamage';
	//WeaponDamageEffect.DamageTypes.AddItem('Electrical');
//	Template.AddTargetEffect(WeaponDamageEffect);

 




	return Template;
}

//static function X2DataTemplate RiposteShot()
//{
	//local X2AbilityTemplate                 Template;
	//local X2AbilityToHitCalc_StatCheck_UnitVsUnit    StatContest;
	//local X2AbilityTarget_Single            SingleTarget;
	//local X2Effect_Stunned					DisorientedEffect;
 //
	//local X2Effect_Stunned				    StunnedEffect;
	//local X2Effect_Stunned					UnconsciousEffect;
//
		//`CREATE_X2ABILITY_TEMPLATE(Template, 'RiposteShot');
//
	//Template.bDontDisplayInAbilitySummary = true;
	//Template.AbilitySourceName = 'eAbilitySource_Standard';
	//Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
//
	//SingleTarget = new class'X2AbilityTarget_Single';
	//SingleTarget.OnlyIncludeTargetsInsideWeaponRange = true;
	//Template.AbilityTargetStyle = SingleTarget;
//
	//Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');      //  ability is activated by another ability that hits
//
	//// Target Conditions
	////
	//Template.AbilityTargetConditions.AddItem(default.LivingTargetUnitOnlyProperty);
	//Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);
//
	//// Shooter Conditions
	////
	//Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
//
	//Template.AddShooterEffectExclusions();
//
	//// This will be a stat contest
	//StatContest = new class'X2AbilityToHitCalc_StatCheck_UnitVsUnit';
	//StatContest.AttackerStat = estat_Will;
	////	StatContest.AttackerStat = eStat_Offense;
	//Template.AbilityToHitCalc = StatContest;
//
	//// On hit effects
	////  Stunned effect for 1 or 2 unblocked hit
	 //
	//DisorientedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	//DisorientedEffect.MinStatContestResult = 0;
	//DisorientedEffect.MaxStatContestResult = 2;
	//DisorientedEffect.bRemoveWhenSourceDies = false;
	//Template.AddTargetEffect(DisorientedEffect);
//
 	////  Stunned effect for 3 or 4 unblocked hit
	//StunnedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	//StunnedEffect.MinStatContestResult = 3;
	//StunnedEffect.MaxStatContestResult = 4;
	//StunnedEffect.bRemoveWhenSourceDies = false;
	//Template.AddTargetEffect(StunnedEffect);
//
 	//UnconsciousEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	//UnconsciousEffect.MinStatContestResult = 5;
	//UnconsciousEffect.MaxStatContestResult = 0;
	//UnconsciousEffect.bRemoveWhenSourceDies = false;
	//Template.AddTargetEffect(UnconsciousEffect);
 //
//
	//Template.bSkipPerkActivationActions = true;
	//Template.bSkipFireAction = true;
	//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//Template.BuildVisualizationFn = StunLancerImpairing_BuildVisualization;
//
	//return Template;
//}
 
 //working below
static function X2DataTemplate RiposteShot()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityToHitCalc_StatCheck_UnitVsUnit    StatContest;
	local X2AbilityTarget_Single            SingleTarget;
	local X2Effect_Stunned					DisorientedEffect;
 
	local X2Effect_Stunned				    StunnedEffect;
	local X2Effect_Stunned					UnconsciousEffect;

		`CREATE_X2ABILITY_TEMPLATE(Template, 'RiposteShot');

	Template.bDontDisplayInAbilitySummary = true;
	Template.AbilitySourceName = 'eAbilitySource_Standard';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;

	SingleTarget = new class'X2AbilityTarget_Single';
	SingleTarget.OnlyIncludeTargetsInsideWeaponRange = true;
	Template.AbilityTargetStyle = SingleTarget;

	Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');      //  ability is activated by another ability that hits

	// Target Conditions
	//
	Template.AbilityTargetConditions.AddItem(default.LivingTargetUnitOnlyProperty);
	Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);

	// Shooter Conditions
	//
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);

	Template.AddShooterEffectExclusions();

	// This will be a stat contest
	StatContest = new class'X2AbilityToHitCalc_StatCheck_UnitVsUnit';
	StatContest.AttackerStat = estat_Will;
	StatContest.DefenderStat = estat_Will;
	//	StatContest.AttackerStat = eStat_Offense;
	Template.AbilityToHitCalc = StatContest;

	// On hit effects
	//  Stunned effect for 1 or 2 unblocked hit
	 
	DisorientedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	DisorientedEffect.MinStatContestResult = 0;
	DisorientedEffect.MaxStatContestResult = 2;
	DisorientedEffect.bRemoveWhenSourceDies = false;
	Template.AddTargetEffect(DisorientedEffect);

 	//  Stunned effect for 3 or 4 unblocked hit
	StunnedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(3, 100);
	StunnedEffect.MinStatContestResult = 3;
	StunnedEffect.MaxStatContestResult = 4;
	StunnedEffect.bRemoveWhenSourceDies = false;
	Template.AddTargetEffect(StunnedEffect);

 	UnconsciousEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(3, 100);
	UnconsciousEffect.MinStatContestResult = 5;
	UnconsciousEffect.MaxStatContestResult = 0;
	UnconsciousEffect.bRemoveWhenSourceDies = false;
	Template.AddTargetEffect(UnconsciousEffect);
 

	Template.bSkipPerkActivationActions = true;
	Template.bSkipFireAction = true;
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = StunLancerImpairing_BuildVisualization;

	return Template;
}
 
//static function X2DataTemplate RiposteShot()
//{
	//local X2AbilityTemplate                 Template;
////	local X2AbilityToHitCalc_StatCheck_UnitVsUnit    StatContest;
	//local X2AbilityTarget_Single            SingleTarget;
////	local X2Effect_Stunned					DisorientedEffect;
	//local X2Condition_UnitStatCheck			UnitStatCheckCondition;
	//local X2Effect_Stunned				    StunnedEffect;
////	local X2Effect_Stunned					UnconsciousEffect;
//
		//`CREATE_X2ABILITY_TEMPLATE(Template, 'RiposteShot');
//
	//Template.bDontDisplayInAbilitySummary = true;
	//Template.AbilitySourceName = 'eAbilitySource_Standard';
	//Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
//
	//SingleTarget = new class'X2AbilityTarget_Single';
	//SingleTarget.OnlyIncludeTargetsInsideWeaponRange = true;
	//Template.AbilityTargetStyle = SingleTarget;
//
	//Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');      //  ability is activated by another ability that hits
//
	//// Target Conditions
	////
	//Template.AbilityTargetConditions.AddItem(default.LivingTargetUnitOnlyProperty);
	//Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);
//
	//// Shooter Conditions
	////
	//Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
//
	//Template.AddShooterEffectExclusions();
//
//
//
//
	//// This will be a stat contest
	////StatContest = new class'X2AbilityToHitCalc_StatCheck_UnitVsUnit';
	////StatContest.AttackerStat = estat_Will;
	//////	StatContest.AttackerStat = eStat_Offense;
	////Template.AbilityToHitCalc = StatContest;
//
	//// On hit effects
	////  Stunned effect for 1 or 2 unblocked hit
	 //
	////DisorientedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	////DisorientedEffect.MinStatContestResult = 0;
	////DisorientedEffect.MaxStatContestResult = 2;
	////DisorientedEffect.bRemoveWhenSourceDies = false;
	////Template.AddTargetEffect(DisorientedEffect);
//
 	////  Stunned effect for 3 or 4 unblocked hit
	//StunnedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	////StunnedEffect.MinStatContestResult = 3;
	////StunnedEffect.MaxStatContestResult = 4;
	//StunnedEffect.bRemoveWhenSourceDies = false;
	//StunnedEffect.DuplicateResponse = eDupe_Ignore;
	//Template.AddTargetEffect(StunnedEffect);
//
	//UnitStatCheckCondition = new class'X2Condition_UnitStatCheck';
	//UnitStatCheckCondition.AddCheckStat(eStat_will, 120, eCheck_LessThan);
	//Template.AbilityTargetConditions.AddItem(UnitStatCheckCondition);
////	StunnedEffect.AbilityTargetConditions.AddItem(UnitStatCheckCondition);
 //
	//Template.bSkipPerkActivationActions = true;
	//Template.bSkipFireAction = true;
	//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//Template.BuildVisualizationFn = StunLancerImpairing_BuildVisualization;
//
	//return Template;
//}

//static function X2DataTemplate RiposteShot6()
//{
	//local X2AbilityTemplate                 Template;
////	local X2AbilityToHitCalc_StatCheck_UnitVsUnit    StatContest;
	//local X2AbilityTarget_Single            SingleTarget;
////	local X2Effect_Stunned					DisorientedEffect;
	//local X2Condition_UnitStatCheck			UnitStatCheckCondition;
	//local X2Effect_Persistent				    StunnedEffect;
////	local X2Effect_Stunned					UnconsciousEffect;
//
		//`CREATE_X2ABILITY_TEMPLATE(Template, 'RiposteShot6');
//
	//Template.bDontDisplayInAbilitySummary = true;
	//Template.AbilitySourceName = 'eAbilitySource_Standard';
	//Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
//
	//SingleTarget = new class'X2AbilityTarget_Single';
	//SingleTarget.OnlyIncludeTargetsInsideWeaponRange = true;
	//Template.AbilityTargetStyle = SingleTarget;
//
	//Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');      //  ability is activated by another ability that hits
//
	//// Target Conditions
	////
	//Template.AbilityTargetConditions.AddItem(default.LivingTargetUnitOnlyProperty);
	//Template.AbilityTargetConditions.AddItem(default.GameplayVisibilityCondition);
//
	//// Shooter Conditions
	////
	//Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
//
	//Template.AddShooterEffectExclusions();
//
//
	//UnitStatCheckCondition = new class'X2Condition_UnitStatCheck';
	//UnitStatCheckCondition.AddCheckStat(eStat_will, 55, eCheck_GreaterThan);
	//Template.AbilityTargetConditions.AddItem(UnitStatCheckCondition);
//
	//// This will be a stat contest
	////StatContest = new class'X2AbilityToHitCalc_StatCheck_UnitVsUnit';
	////StatContest.AttackerStat = estat_Will;
	//////	StatContest.AttackerStat = eStat_Offense;
	////Template.AbilityToHitCalc = StatContest;
//
	//// On hit effects
	////  Stunned effect for 1 or 2 unblocked hit
	 //
	////DisorientedEffect = class'X2StatusEffects'.static.CreateStunnedStatusEffect(2, 100);
	////DisorientedEffect.MinStatContestResult = 0;
	////DisorientedEffect.MaxStatContestResult = 2;
	////DisorientedEffect.bRemoveWhenSourceDies = false;
	////Template.AddTargetEffect(DisorientedEffect);
//
 	////  Stunned effect for 3 or 4 unblocked hit
	//StunnedEffect = class'X2StatusEffects'.static.CreateDisorientedStatusEffect();
	//StunnedEffect.BuildPersistentEffect(4,, false,,eGameRule_PlayerTurnBegin);
	////StunnedEffect.MinStatContestResult = 3;
	////StunnedEffect.MaxStatContestResult = 4;
	//StunnedEffect.bRemoveWhenSourceDies = false;
	//Template.AddTargetEffect(StunnedEffect);
 //
	//Template.bSkipPerkActivationActions = true;
	//Template.bSkipFireAction = true;
	//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//Template.BuildVisualizationFn = StunLancerImpairing_BuildVisualization;
//
	//return Template;
//}

simulated function StunLancerImpairing_BuildVisualization(XComGameState VisualizeGameState, out array<VisualizationTrack> OutVisualizationTracks)
{
	local XComGameStateHistory History;
	local XComGameStateContext_Ability  Context;
	local StateObjectReference InteractingUnitRef;
	local VisualizationTrack EmptyTrack;
	local VisualizationTrack BuildTrack;

	History = `XCOMHISTORY;

	Context = XComGameStateContext_Ability(VisualizeGameState.GetContext());
	InteractingUnitRef = Context.InputContext.SourceObject;

	//Configure the visualization track for the shooter
	//****************************************************************************************
	BuildTrack = EmptyTrack;
	BuildTrack.StateObject_OldState = History.GetGameStateForObjectID(InteractingUnitRef.ObjectID, eReturnType_Reference, VisualizeGameState.HistoryIndex - 1);
	BuildTrack.StateObject_NewState = VisualizeGameState.GetGameStateForObjectID(InteractingUnitRef.ObjectID);
	BuildTrack.TrackActor = History.GetVisualizer(InteractingUnitRef.ObjectID);

	class'X2Action_AbilityPerkStart'.static.AddToVisualizationTrack(BuildTrack, Context);
	class'X2Action_AbilityPerkEnd'.static.AddToVisualizationTrack(BuildTrack, Context);

	OutVisualizationTracks.AddItem(BuildTrack);
}


static function ImpairingAbilityEffectTriggeredVisualization(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	local XComGameStateContext_Ability AbilityContext;
	local XComGameStateContext Context;
	local XComGameStateContext_Ability TestAbilityContext;
	local int i, j;
	local XComGameStateHistory History;
	local bool bAbilityWasSuccess;
	local X2AbilityTemplate AbilityTemplate;
	local X2VisualizerInterface TargetVisualizerInterface;

	if( (EffectApplyResult != 'AA_Success') || (XComGameState_Unit(BuildTrack.StateObject_NewState) == none) )
	{
		return;
	}

	Context = VisualizeGameState.GetContext();
	AbilityContext = XComGameStateContext_Ability(Context);

	if( AbilityContext.EventChainStartIndex != 0 )
	{
		History = `XCOMHISTORY;

		// This GameState is part of a chain, which means there may be a stun to the target
		for( i = AbilityContext.EventChainStartIndex; !Context.bLastEventInChain; ++i )
		{
			Context = History.GetGameStateFromHistory(i).GetContext();

			TestAbilityContext = XComGameStateContext_Ability(Context);
			bAbilityWasSuccess = (TestAbilityContext != none) && class'XComGameStateContext_Ability'.static.IsHitResultHit(TestAbilityContext.ResultContext.HitResult);

			if( bAbilityWasSuccess &&
				TestAbilityContext.InputContext.AbilityTemplateName == 'RiposteShot' &&
				TestAbilityContext.InputContext.SourceObject.ObjectID == AbilityContext.InputContext.SourceObject.ObjectID &&
				TestAbilityContext.InputContext.PrimaryTarget.ObjectID == AbilityContext.InputContext.PrimaryTarget.ObjectID )
			{
				// The Melee Impairing Ability has been found with the same source and target
				// Move that ability's visualization forward to this track
				AbilityTemplate = class'XComGameState_Ability'.static.GetMyTemplateManager().FindAbilityTemplate(TestAbilityContext.InputContext.AbilityTemplateName);

				for( j = 0; j < AbilityTemplate.AbilityTargetEffects.Length; ++j )
				{
					AbilityTemplate.AbilityTargetEffects[j].AddX2ActionsForVisualization(Context.AssociatedState, BuildTrack, TestAbilityContext.FindTargetEffectApplyResult(AbilityTemplate.AbilityTargetEffects[j]));
				}

				TargetVisualizerInterface = X2VisualizerInterface(BuildTrack.TrackActor);
				if (TargetVisualizerInterface != none)
				{
					TargetVisualizerInterface.BuildAbilityEffectsVisualization(Context.AssociatedState, BuildTrack);
				}
			}
		}
	}
}

 


static function X2AbilityTemplate IsmsSelfSufficient()
{
	local X2AbilityTemplate         Template;

	Template = PurePassive('IsmsSelfSufficient', "img:///UILibrary_PerkIcons.UIPerk_LeadByExample");
	Template.AdditionalAbilities.AddItem('IsmsSelfSufficient2Trigger');
	Template.AdditionalAbilities.AddItem('IsmsSelfSufficient3Trigger');
	Template.AdditionalAbilities.AddItem('IsmsSelfSufficient4Trigger');
	Template.AdditionalAbilities.AddItem('IsmsSelfSufficient5Trigger');
	Template.AdditionalAbilities.AddItem('IsmsSelfSufficientWill');
	
	Template.bCrossClassEligible = true;

	return Template;
}

static function X2AbilityTemplate IsmsSelfSufficient2Trigger()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityTrigger_EventListener    Trigger;
  	local X2Condition_UnitValue				ShooterCondition;	  
	local X2Effect_RemoveEffects			RemoveEffects;
	local X2Effect_RemoveEffectsByDamageType RemoveEffectTypes;
 	local X2Effect_ApplyMedikitHeal         MedikitHeal;
	local X2Condition_UnitEffects			ShooterEffects;
					 
	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsSelfSufficient2Trigger');

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;

	Trigger = new class'X2AbilityTrigger_EventListener';
	Trigger.ListenerData.Deferral = ELD_OnStateSubmitted;
	Trigger.ListenerData.EventID = 'PlayerTurnEnded';
	Trigger.ListenerData.Filter = eFilter_Player;
 	Trigger.ListenerData.EventFn = class'XComGameState_Ability'.static.AbilityTriggerEventListener_Self;
	Template.AbilityTriggers.AddItem(Trigger);

	
	ShooterCondition = new class'X2Condition_UnitValue';
 	ShooterCondition.AddCheckValue('NonMoveActionsThisTurn', 0);
 	Template.AbilityShooterConditions.AddItem(ShooterCondition);

	ShooterEffects = new class'X2Condition_UnitEffects';
 	ShooterEffects.AddRequireEffect(class'X2StatusEffects'.default. MarkedName, 'AA_UnitIsNotMarked'); 
	
 	Template.AbilityShooterConditions.AddItem(ShooterEffects);
	
	RemoveEffects = new class'X2Effect_RemoveEffects';
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.BurningName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.AcidBurningName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default. MarkedName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.PoisonedName);
 	Template.AddTargetEffect(RemoveEffects);

	RemoveEffectTypes = new class'X2Effect_RemoveEffectsByDamageType';
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Fire');
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Poison');
	RemoveEffectTypes.DamageTypesToRemove.AddItem(class'X2Effect_ParthenogenicPoison'.default.ParthenogenicPoisonType);
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Acid');
	 
 
	MedikitHeal = new class'X2Effect_ApplyMedikitHeal';
	MedikitHeal.PerUseHP = 2;
 //	MedikitHeal.IncreasedHealProject = 'BattlefieldMedicine';
//	MedikitHeal.IncreasedPerUseHP = class'X2Ability_DefaultAbilitySet'.default.NANOMEDIKIT_PERUSEHP;
	Template.AddTargetEffect(MedikitHeal);	

 	Template.CustomSelfFireAnim = 'FF_FireMedkitSelf';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
		
	Template.PostActivationEvents.AddItem('ItemRecalled');
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;  
 	return Template;
}
 
 static function X2AbilityTemplate IsmsSelfSufficient3Trigger()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityTrigger_EventListener    Trigger;
  	local X2Condition_UnitValue				ShooterCondition;	 //here
	local X2Effect_RemoveEffects			RemoveEffects;
	local X2Effect_RemoveEffectsByDamageType RemoveEffectTypes;
 	local X2Effect_ApplyMedikitHeal         MedikitHeal;
	local X2Condition_UnitEffects			ShooterEffects;
					 
	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsSelfSufficient3Trigger');

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;

	Trigger = new class'X2AbilityTrigger_EventListener';
	Trigger.ListenerData.Deferral = ELD_OnStateSubmitted;
	Trigger.ListenerData.EventID = 'PlayerTurnEnded';
	Trigger.ListenerData.Filter = eFilter_Player;
 	Trigger.ListenerData.EventFn = class'XComGameState_Ability'.static.AbilityTriggerEventListener_Self;
	Template.AbilityTriggers.AddItem(Trigger);

	
	ShooterCondition = new class'X2Condition_UnitValue';
 	ShooterCondition.AddCheckValue('NonMoveActionsThisTurn', 0);
 	Template.AbilityShooterConditions.AddItem(ShooterCondition);

	ShooterEffects = new class'X2Condition_UnitEffects';
 	ShooterEffects.AddRequireEffect(class'X2StatusEffects'.default.AcidBurningName, 'AA_UnitIsNotAcidBurning');
	
 	Template.AbilityShooterConditions.AddItem(ShooterEffects);
	
	RemoveEffects = new class'X2Effect_RemoveEffects';
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.BurningName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.AcidBurningName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default. MarkedName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.PoisonedName);
 	Template.AddTargetEffect(RemoveEffects);

	RemoveEffectTypes = new class'X2Effect_RemoveEffectsByDamageType';
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Fire');
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Poison');
	RemoveEffectTypes.DamageTypesToRemove.AddItem(class'X2Effect_ParthenogenicPoison'.default.ParthenogenicPoisonType);
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Acid');
	 
 
	MedikitHeal = new class'X2Effect_ApplyMedikitHeal';
	MedikitHeal.PerUseHP = 2;
// 	MedikitHeal.IncreasedHealProject = 'BattlefieldMedicine';
//	MedikitHeal.IncreasedPerUseHP = class'X2Ability_DefaultAbilitySet'.default.NANOMEDIKIT_PERUSEHP;
	Template.AddTargetEffect(MedikitHeal);	

 	Template.CustomSelfFireAnim = 'FF_FireMedkitSelf';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
		
	Template.PostActivationEvents.AddItem('ItemRecalled');
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;  
 	return Template;
}

static function X2AbilityTemplate IsmsSelfSufficient4Trigger()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityTrigger_EventListener    Trigger;
  	local X2Condition_UnitValue				ShooterCondition;	 //here
	local X2Effect_RemoveEffects			RemoveEffects;
	local X2Effect_RemoveEffectsByDamageType RemoveEffectTypes;
 	local X2Effect_ApplyMedikitHeal         MedikitHeal;
	local X2Condition_UnitEffects			ShooterEffects;
					 
	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsSelfSufficient4Trigger');

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;

	Trigger = new class'X2AbilityTrigger_EventListener';
	Trigger.ListenerData.Deferral = ELD_OnStateSubmitted;
	Trigger.ListenerData.EventID = 'PlayerTurnEnded';
	Trigger.ListenerData.Filter = eFilter_Player;
 	Trigger.ListenerData.EventFn = class'XComGameState_Ability'.static.AbilityTriggerEventListener_Self;
	Template.AbilityTriggers.AddItem(Trigger);

	
	ShooterCondition = new class'X2Condition_UnitValue';
 	ShooterCondition.AddCheckValue('NonMoveActionsThisTurn', 0);
 	Template.AbilityShooterConditions.AddItem(ShooterCondition);

	ShooterEffects = new class'X2Condition_UnitEffects';
 	ShooterEffects.AddRequireEffect(class'X2StatusEffects'.default.PoisonedName, 'AA_UnitIsNotPoisoned');
	
 	Template.AbilityShooterConditions.AddItem(ShooterEffects);
	
	RemoveEffects = new class'X2Effect_RemoveEffects';
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.BurningName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.AcidBurningName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default. MarkedName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.PoisonedName);
 	Template.AddTargetEffect(RemoveEffects);

	RemoveEffectTypes = new class'X2Effect_RemoveEffectsByDamageType';
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Fire');
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Poison');
	RemoveEffectTypes.DamageTypesToRemove.AddItem(class'X2Effect_ParthenogenicPoison'.default.ParthenogenicPoisonType);
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Acid');
	 
  

	MedikitHeal = new class'X2Effect_ApplyMedikitHeal';
	MedikitHeal.PerUseHP = 2;
//	MedikitHeal.IncreasedHealProject = 'BattlefieldMedicine';
//	MedikitHeal.IncreasedPerUseHP = class'X2Ability_DefaultAbilitySet'.default.NANOMEDIKIT_PERUSEHP;
	Template.AddTargetEffect(MedikitHeal);	

	Template.CustomSelfFireAnim = 'FF_FireMedkitSelf';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
		
	Template.PostActivationEvents.AddItem('ItemRecalled');
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization; 
	return Template;
}

static function X2AbilityTemplate IsmsSelfSufficient5Trigger()
{
	local X2AbilityTemplate                 Template;
	local X2AbilityTrigger_EventListener    Trigger;
	local X2Condition_UnitValue				ShooterCondition;	 
	local X2Effect_RemoveEffects			RemoveEffects;
	local X2Effect_RemoveEffectsByDamageType RemoveEffectTypes;
	local X2Effect_ApplyMedikitHeal         MedikitHeal;
	local X2Condition_UnitEffects			ShooterEffects;
					 
	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsSelfSufficient5Trigger');

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;

	Trigger = new class'X2AbilityTrigger_EventListener';
	Trigger.ListenerData.Deferral = ELD_OnStateSubmitted;
	Trigger.ListenerData.EventID = 'PlayerTurnEnded';
	Trigger.ListenerData.Filter = eFilter_Player;
 	Trigger.ListenerData.EventFn = class'XComGameState_Ability'.static.AbilityTriggerEventListener_Self;
	Template.AbilityTriggers.AddItem(Trigger);

	
	ShooterCondition = new class'X2Condition_UnitValue';
 	ShooterCondition.AddCheckValue('NonMoveActionsThisTurn', 0);
 	Template.AbilityShooterConditions.AddItem(ShooterCondition);

	ShooterEffects = new class'X2Condition_UnitEffects';
 	ShooterEffects.AddRequireEffect(class'X2StatusEffects'.default.BurningName, 'AA_UnitIsNotBurning');
	
 	Template.AbilityShooterConditions.AddItem(ShooterEffects);
	
	RemoveEffects = new class'X2Effect_RemoveEffects';
 	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.BurningName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.AcidBurningName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default. MarkedName);
	RemoveEffects.EffectNamesToRemove.AddItem(class'X2StatusEffects'.default.PoisonedName);
	Template.AddTargetEffect(RemoveEffects);

	RemoveEffectTypes = new class'X2Effect_RemoveEffectsByDamageType';
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Fire');
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Poison');
	RemoveEffectTypes.DamageTypesToRemove.AddItem(class'X2Effect_ParthenogenicPoison'.default.ParthenogenicPoisonType);
	RemoveEffectTypes.DamageTypesToRemove.AddItem('Acid');
	 

	MedikitHeal = new class'X2Effect_ApplyMedikitHeal';
	MedikitHeal.PerUseHP = 2;
//	MedikitHeal.IncreasedHealProject = 'BattlefieldMedicine';
//	MedikitHeal.IncreasedPerUseHP = class'X2Ability_DefaultAbilitySet'.default.NANOMEDIKIT_PERUSEHP;
	Template.AddTargetEffect(MedikitHeal);	

	Template.CustomSelfFireAnim = 'FF_FireMedkitSelf';
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
		
	Template.PostActivationEvents.AddItem('ItemRecalled');
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization; 
	return Template;
}

static function X2AbilityTemplate IsmsSelfSufficientWill()
{
	local X2AbilityTemplate						Template;
 	local X2Effect_PersistentStatChange     PersistentStatChangeEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsSelfSufficientWill');

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = EAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_secondaryheart";

	Template.AbilityToHitCalc = default.DeadEye;
	Template.AbilityTargetStyle = default.SelfTarget;
	Template.AbilityTriggers.AddItem(default.UnitPostBeginPlayTrigger);
	
	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Will, 5);
	PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Passive, "Self-Sufficient", "The Rogue can remove Burning, Poison, Acid, or Marked conditions if no ability has been used this turn", Template.IconImage,,,Template.AbilitySourceName);
	Template.AddTargetEffect(PersistentStatChangeEffect);
 
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//  NOTE: No visualization on purpose!


	return Template;
}

//static function X2AbilityTemplate IsmsTasteOfDeath()
//{
	//local X2AbilityTemplate								Template;
	//local X2AbilityTargetStyle							TargetStyle;
	//local X2AbilityTrigger								Trigger;
	//local X2Effect_Isms_TasteOfDeath			TODEffect;
	//
 	//`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsTasteOfDeath');
//
	//Template.AbilitySourceName = 'eAbilitySource_Perk';
	//Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
	//Template.Hostility = eHostility_Neutral;
	//Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_executioner";
//
	//Template.AbilityToHitCalc = default.DeadEye;
//
	//TargetStyle = new class'X2AbilityTarget_Self';
	//Template.AbilityTargetStyle = TargetStyle;
//
	//Trigger = new class'X2AbilityTrigger_UnitPostBeginPlay';
	//Template.AbilityTriggers.AddItem(Trigger);
//
	//TODEffect = new class'X2Effect_Isms_TasteOfDeath';
	//TODEffect.EffectName = 'Isms_TasteOfDeathr';
	//TODEffect.BuildPersistentEffect(1, true, false);
	//TODEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	//TODEffect.DuplicateResponse = eDupe_Ignore;
	//Template.AddTargetEffect(TODEffect);
//
	//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	////  NOTE: No visualization on purpose!
//
	//Template.bCrossClassEligible = true;
//
	//return Template;
//}




static function X2AbilityTemplate IsmsTODStart()
{
	local X2AbilityTemplate								Template;
	local X2AbilityTargetStyle							TargetStyle;
	local X2AbilityTrigger								Trigger;
	local X2Effect_Isms_OpenerStance					TODEffect;
	local X2Condition_PlayerTurns					    TurnsCondition;
	 
	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsTODStart');

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_muton_execute";

	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;

	Trigger = new class'X2AbilityTrigger_UnitPostBeginPlay';
	Template.AbilityTriggers.AddItem(Trigger);

	TurnsCondition = new class'X2Condition_PlayerTurns';
	TurnsCondition.NumTurnsCheck.CheckType = eCheck_LessThan;
	TurnsCondition.NumTurnsCheck.Value = 2;
	TODEffect.TargetConditions.AddItem(TurnsCondition);

	TODEffect = new class'X2Effect_Isms_OpenerStance';
	TODEffect.EffectName = 'Isms_OpenerStance';
	TODEffect.BuildPersistentEffect(1, true, false);
	TODEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), "img:///UILibrary_PerkIcons.UIPerk_quadricepshypertrophy",,,Template.AbilitySourceName);
	TODEffect.DuplicateResponse = eDupe_Ignore;
	Template.AddTargetEffect(TODEffect);

	Template.AdditionalAbilities.AddItem('IsmsCloserStance');
	Template.AdditionalAbilities.AddItem('IsmsOpenerStance');
	Template.AdditionalAbilities.AddItem('IsmsTODCooldown');


	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	
	Template.bCrossClassEligible = false;

	return Template;
}

static function X2AbilityTemplate IsmsCloserStance()
{
	local X2AbilityTemplate								Template;
	local X2AbilityTargetStyle							TargetStyle;
	local X2AbilityTrigger								Trigger;
	local X2Effect_Isms_CloserStance					TODEffect;
	local X2Condition_UnitEffects						ShooterEffects; 
	local X2Condition_UnitEffects						CooldownRequirement; 
	local X2Effect_RemoveEffects						RemoveEffects;
	local X2Effect_TODCooldown							CooldownEffect;
	local X2Effect_DelayedAbilityActivation				DelayedCooldown;
	local X2AbilityCost_ActionPoints					ActionPointCost;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsCloserStance');

	Template.AbilitySourceName = 'eAbilitySource_Commander';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_ShowIfAvailable;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_hithurts";

	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;

	Trigger = new class'X2AbilityTrigger_PlayerInput';
	Template.AbilityTriggers.AddItem(Trigger);
 
 	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = false;   //  this will guarantee the unit has at least 1 action point
	ActionPointCost.bFreeCost = true;           //  ReserveActionPoints effect will take all action points away
	Template.AbilityCosts.AddItem(ActionPointCost);  
 

	ShooterEffects = new class'X2Condition_UnitEffects';
 	ShooterEffects.AddRequireEffect('Isms_OpenerStance', 'AA_UnitIsNotOpener');
	//ShooterEffects.AddExcludeEffect('TODCooldown', 'AA_UnitIsNotTODCooldown');
	Template.AbilityShooterConditions.AddItem(ShooterEffects);

	CooldownRequirement = new class'X2Condition_UnitEffects';
 	CooldownRequirement.AddExcludeEffect('TODCooldown', 'AA_UnitIsNotTODCooldown');
	Template.AbilityShooterConditions.AddItem(CooldownRequirement);

	RemoveEffects = new class'X2Effect_RemoveEffects';
	RemoveEffects.EffectNamesToRemove.AddItem('Isms_CloserStance');
	RemoveEffects.EffectNamesToRemove.AddItem('Isms_OpenerStance');
	RemoveEffects.EffectNamesToRemove.AddItem('TODCooldown');
 	Template.AddTargetEffect(RemoveEffects);

	TODEffect = new class'X2Effect_Isms_CloserStance';
	TODEffect.EffectName = 'Isms_CloserStance';
	TODEffect.BuildPersistentEffect(1, true, false);
	TODEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	TODEffect.DuplicateResponse = eDupe_Ignore;
	Template.AddTargetEffect(TODEffect);

	CooldownEffect = new class'X2Effect_TODCooldown';
	CooldownEffect.EffectName = 'TODCooldown';
	CooldownEffect.BuildPersistentEffect(2, false, false);
	CooldownEffect.iNumTurns = 2;
	CooldownEffect.bInfiniteDuration = false;
	//CooldownEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), "img:///UILibrary_PerkIcons.UIPerk_stun",,,Template.AbilitySourceName);
	CooldownEffect.DuplicateResponse = eDupe_ignore;  //ignore?
	Template.AddTargetEffect(CooldownEffect);

	DelayedCooldown = new class 'X2Effect_DelayedAbilityActivation';
	DelayedCooldown.BuildPersistentEffect(5, false, false, , eGameRule_PlayerTurnBegin);
	DelayedCooldown.EffectName = 'DelayedCooldown';
	DelayedCooldown.TriggerEventName = 'DelayedCooldown';
//	DelayedCooldown.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, true, , Template.AbilitySourceName);
	Template.AddShooterEffect(DelayedCooldown);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	
	Template.bCrossClassEligible = false;

	return Template;
}

static function X2AbilityTemplate IsmsOpenerStance()
{
	local X2AbilityTemplate								Template;
	local X2AbilityTargetStyle							TargetStyle;
	local X2AbilityTrigger								Trigger;
	local X2Effect_Isms_OpenerStance					TODEffect;
 	 
	local X2Condition_UnitEffects						ShooterEffects; 
	local X2Condition_UnitEffects						CooldownRequirement; 
	local X2Effect_RemoveEffects						RemoveEffects;
	local X2Effect_TODCooldown							CooldownEffect;
	local X2Effect_DelayedAbilityActivation				DelayedCooldown;
	local X2AbilityCost_ActionPoints					ActionPointCost;


	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsOpenerStance');

	Template.AbilitySourceName = 'eAbilitySource_Commander';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_ShowIfAvailable;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_quadricepshypertrophy";

	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;

	Trigger = new class'X2AbilityTrigger_PlayerInput';
	Template.AbilityTriggers.AddItem(Trigger);
  
	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = false;   //  this will guarantee the unit has at least 1 action point
	ActionPointCost.bFreeCost = true;           //  ReserveActionPoints effect will take all action points away
	Template.AbilityCosts.AddItem(ActionPointCost);  

	ShooterEffects = new class'X2Condition_UnitEffects';
 	ShooterEffects.AddRequireEffect('Isms_CloserStance', 'AA_UnitIsNotOpener');
//	ShooterEffects.AddExcludeEffect('TODCooldown', 'AA_UnitIsNotTODCooldown');
	Template.AbilityShooterConditions.AddItem(ShooterEffects);

	
	CooldownRequirement = new class'X2Condition_UnitEffects';
 	CooldownRequirement.AddExcludeEffect('TODCooldown', 'AA_UnitIsNotTODCooldown');
	Template.AbilityShooterConditions.AddItem(CooldownRequirement);

	RemoveEffects = new class'X2Effect_RemoveEffects';
	RemoveEffects.EffectNamesToRemove.AddItem('Isms_CloserStance');
	RemoveEffects.EffectNamesToRemove.AddItem('Isms_OpenerStance');
	RemoveEffects.EffectNamesToRemove.AddItem('TODCooldown');
 	Template.AddTargetEffect(RemoveEffects);

	TODEffect = new class'X2Effect_Isms_OpenerStance';
	TODEffect.EffectName = 'Isms_OpenerStance';
	TODEffect.BuildPersistentEffect(1, true, false);
	TODEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	TODEffect.DuplicateResponse = eDupe_Ignore;
	Template.AddTargetEffect(TODEffect);

	CooldownEffect = new class'X2Effect_TODCooldown';
	CooldownEffect.EffectName = 'TODCooldown';
	CooldownEffect.iNumTurns = 2;
	CooldownEffect.bInfiniteDuration = false;
	CooldownEffect.BuildPersistentEffect(2, false, false); //(5, false, false, , eGameRule_PlayerTurnBegin)
	//CooldownEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), "img:///UILibrary_PerkIcons.UIPerk_stun",,,Template.AbilitySourceName);
	CooldownEffect.DuplicateResponse = eDupe_ignore;  //ignore?
	Template.AddTargetEffect(CooldownEffect);

	DelayedCooldown = new class 'X2Effect_DelayedAbilityActivation';
	DelayedCooldown.BuildPersistentEffect(5, false, false, , eGameRule_PlayerTurnBegin);
	DelayedCooldown.EffectName = 'DelayedCooldown';
	DelayedCooldown.TriggerEventName = 'DelayedCooldown';
	//DelayedCooldown.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage, true, , Template.AbilitySourceName);
	Template.AddShooterEffect(DelayedCooldown);


	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	
	Template.bCrossClassEligible = false;

	return Template;
}

static function X2AbilityTemplate IsmsTODCooldown()
{
	local X2AbilityTemplate								Template;
	local X2AbilityTargetStyle							TargetStyle;
	local X2AbilityTrigger								Trigger;
  	local X2Effect_TODCooldown							CooldownEffect;
	local X2Effect_RemoveEffects						RemoveEffects;
	local X2AbilityTrigger_EventListener				 EventListener;
				 
	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsTODCooldown');

	Template.AbilitySourceName = 'eAbilitySource_Perk';
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;
	Template.Hostility = eHostility_Neutral;
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_executioner";

	EventListener = new class'X2AbilityTrigger_EventListener';
	EventListener.ListenerData.Deferral = ELD_OnStateSubmitted;
	EventListener.ListenerData.EventID = 'DelayedCooldown';
	EventListener.ListenerData.Filter = eFilter_Unit;
	EventListener.ListenerData.EventFn = class'XComGameState_Ability'.static.AbilityTriggerEventListener_SelfIgnoreCache;
	Template.AbilityTriggers.AddItem(EventListener);



	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;

	Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');

	RemoveEffects = new class'X2Effect_RemoveEffects';
 	RemoveEffects.EffectNamesToRemove.AddItem('TODCooldown');
 	Template.AddTargetEffect(RemoveEffects);
  
  	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	
	Template.bCrossClassEligible = false;

	return Template;
}


//You can then consume all stacks to heal 15-90% of missing HP or deal +1-6 dmg on a single attack.
//static function X2AbilityTemplate IsmsSoulCollector()
//{
	//local X2AbilityTemplate								Template;
	//local X2AbilityTargetStyle							TargetStyle;
	//local X2AbilityTrigger								Trigger;
	//local X2Effect_RemoveEffects						RemoveEffects;
	//local X2Effect_ImmediateAbilityActivation ImpairingAbilityEffect;
	//local X2AbilityCharges                      Charges;
	//local X2AbilityCost_ActionPoints    ActionPointCost;
//
	//`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsSoulCollector');
	//Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_ShowIfAvailable;	
	//Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_executioner";
	//Template.Hostility = eHostility_Neutral;
	//Template.AbilityConfirmSound = "TacticalUI_Activate_Ability_Wraith_Armor";
	//Template.AbilitySourceName = 'eAbilitySource_Psionic';
//
	//Template.AbilityToHitCalc = default.DeadEye;
	//TargetStyle = new class'X2AbilityTarget_Self';
	//Template.AbilityTargetStyle = TargetStyle;
	//
 	//Trigger = new class'X2AbilityTrigger_PlayerInput';
	//Template.AbilityTriggers.AddItem(Trigger);
//
 	//Template.AbilityCosts.AddItem(new class'X2AbilityCost_Charges');
 	//Charges = new class'X2AbilityCharges';
	//Charges.InitialCharges = 2; 
	//Template.AbilityCharges = Charges;
//
	//ActionPointCost = new class'X2AbilityCost_ActionPoints';
	//ActionPointCost.iNumPoints = 1;
	//ActionPointCost.bConsumeAllPoints = false;
	//Template.AbilityCosts.AddItem(ActionPointCost);
//
	//RemoveEffects = new class'X2Effect_RemoveEffects';
	//RemoveEffects.EffectNamesToRemove.AddItem('Isms_SoulCollector');
	//RemoveEffects.bcleanse = false;  //if false = heal, if true = no heal.
 	//Template.AddTargetEffect(RemoveEffects);
//
	//ImpairingAbilityEffect = new class 'X2Effect_ImmediateAbilityActivation';
	//ImpairingAbilityEffect.BuildPersistentEffect(1, false, true, , eGameRule_PlayerTurnBegin);
	//ImpairingAbilityEffect.EffectName = 'IsmsSoulCollector2';
	//ImpairingAbilityEffect.AbilityName = 'IsmsSoulCollector2';
	//ImpairingAbilityEffect.bRemoveWhenTargetDies = true;
	//Template.AddTargetEffect(ImpairingAbilityEffect);
//
 	//Template.bShowActivation = true;
	////Template.CustomFireAnim = 'HL_Psi_SelfCast';
	//Template.ActivationSpeech = 'CombatStim';
	////Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	//Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
	//Template.CinescriptCameraType = "Psionic_FireAtUnit";
	////Template.AdditionalAbilities.AddItem('IsmsSoulCollector2');
	////Template.AdditionalAbilities.AddItem('IsmsSoulCollector3');
	//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
//
	//return Template;
//}

  
	static function X2AbilityTemplate IsmsSoulCollector2()
{
	local X2AbilityTemplate								Template;
	local X2AbilityTargetStyle							TargetStyle;
	local X2Effect_Isms_SoulCollector				TrackerEffect;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsSoulCollector2');
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;	
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_executioner";
	Template.Hostility = eHostility_Neutral;
	
	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;
	
	Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');
 
 	TrackerEffect = new class'X2Effect_Isms_SoulCollector';
	TrackerEffect.EffectName = 'Isms_SoulCollector';
	TrackerEffect.bCanTickEveryAction = true;
	TrackerEffect.BuildPersistentEffect(1, true, false);
//	TrackerEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	TrackerEffect.DuplicateResponse = eDupe_ignore;
	Template.AddTargetEffect(TrackerEffect);

  	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.EffectName = 'RushofBattleMax';
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Mobility, 6);
//	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Dodge, 18);
	PersistentStatChangeEffect.DuplicateREsponse = eDupe_Refresh;
	Template.AddTargetEffect(PersistentStatChangeEffect);

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	return Template;
}

static function X2AbilityTemplate IsmsSoulCollector3()
{
	local X2AbilityTemplate								Template;
	local X2AbilityTargetStyle							TargetStyle;
	local X2AbilityTrigger								Trigger;
	local X2Effect_Isms_SoulCollector				TrackerEffect;
	local X2Condition_PlayerTurns					    TurnsCondition;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;

	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsSoulCollector3');
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;	
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_Executioner";
	Template.Hostility = eHostility_Neutral;
	Template.AbilitySourceName = 'eAbilitySource_Perk';

	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;
	

	Trigger = new class'X2AbilityTrigger_UnitPostBeginPlay';
	Template.AbilityTriggers.AddItem(Trigger);
 

	TurnsCondition = new class'X2Condition_PlayerTurns';
	TurnsCondition.NumTurnsCheck.CheckType = eCheck_LessThan;
	TurnsCondition.NumTurnsCheck.Value = 2;
	TrackerEffect.TargetConditions.AddItem(TurnsCondition);
 
 	TrackerEffect = new class'X2Effect_Isms_SoulCollector';
	TrackerEffect.EffectName = 'Isms_SoulCollector';
	TrackerEffect.BuildPersistentEffect(1, true, true, false, eGameRule_PlayerTurnBegin);
	TrackerEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	TrackerEffect.DuplicateResponse = eDupe_ignore;
	Template.AddTargetEffect(TrackerEffect);

  	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.EffectName = 'RushofBattleMax';
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Mobility, 6);
//	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Dodge, 18);
	//PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Passive, "First One", Template.GetMyLongDescription(), Template.IconImage, true,,Template.AbilitySourceName);
	PersistentStatChangeEffect.DuplicateREsponse = eDupe_Refresh;
	Template.AddTargetEffect(PersistentStatChangeEffect);


	Template.AdditionalAbilities.AddItem('IsmsSoulCollector2');
	Template.AdditionalAbilities.AddItem('IsmsNBK'); 
	Template.AdditionalAbilities.AddItem('IsmsNBK2'); 

	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	return Template;
}

 static function X2AbilityTemplate IsmsNBK()
{
	local X2AbilityTemplate								Template;
//	local X2AbilityTargetStyle							TargetStyle;
	local X2AbilityTrigger								Trigger;
	local X2AbilityToHitCalc_StandardAim    ToHitCalc;
	local X2Effect_RemoveEffects						RemoveEffects;
	local X2Effect_ImmediateAbilityActivation ImpairingAbilityEffect;
	local X2AbilityCharges                      Charges;
	local X2AbilityCost_ActionPoints    ActionPointCost;
	local X2Condition_Visibility            TargetVisibilityCondition;
	local X2Condition_UnitProperty          ShooterCondition;	 //here
	local X2AbilityCost_Ammo                AmmoCost;
    local X2AbilityCooldown				 Cooldown;
 
	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsNBK');


	Template.AdditionalAbilities.AddItem('IsmsNBK2'); 
	Template.ConcealmentRule = eConceal_Never;
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_AlwaysShow;	
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_executioner";
	Template.Hostility = eHostility_Offensive;
	Template.AbilityConfirmSound = "TacticalUI_Activate_Ability_Wraith_Armor";
	Template.AbilitySourceName = 'eAbilitySource_Psionic';

	ToHitCalc = new class'X2AbilityToHitCalc_StandardAim';
	Template.AbilityToHitCalc = ToHitCalc;

	//Template.Template.LocDefaultPrimaryWeapon = true;

	Template.TargetingMethod = class'X2TargetingMethod_OverTheShoulder';
	Template.bUsesFiringCamera = true;
	Template.CinescriptCameraType = "StandardGunFiring";

	AmmoCost = new class'X2AbilityCost_Ammo';
	AmmoCost.iAmmo = 1;
	Template.AbilityCosts.AddItem(AmmoCost);
		
 	Trigger = new class'X2AbilityTrigger_PlayerInput';
	Template.AbilityTriggers.AddItem(Trigger);

 	Template.AbilityCosts.AddItem(new class'X2AbilityCost_Charges');
 	Charges = new class'X2AbilityCharges';
	Charges.InitialCharges = 1; 
	Template.AbilityCharges = Charges;

    Cooldown = new class'X2AbilityCooldown';
    Cooldown.iNumTurns = 5;
    Template.AbilityCooldown = Cooldown;

	ActionPointCost = new class'X2AbilityCost_ActionPoints';
	ActionPointCost.iNumPoints = 1;
	ActionPointCost.bConsumeAllPoints = true;
	Template.AbilityCosts.AddItem(ActionPointCost);

	Template.AbilityTargetStyle = default.SimpleSingleTarget;
	Template.AbilityTriggers.AddItem(default.PlayerInputTrigger);

	Template.AbilityShooterConditions.AddItem(default.LivingShooterProperty);	
	ShooterCondition = new class'X2Condition_UnitProperty';
	//ShooterCondition.IsUnspotted = true;  //fix this here needs to only be activateable while concealed.
	Template.AbilityShooterConditions.AddItem(ShooterCondition);

	TargetVisibilityCondition = new class'X2Condition_Visibility';
	TargetVisibilityCondition.bRequireGameplayVisible = true;
	TargetVisibilityCondition.bAllowSquadsight = true;
	Template.AbilityTargetConditions.AddItem(TargetVisibilityCondition);
	Template.AbilityTargetConditions.AddItem(default.LivingHostileTargetProperty);

	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.HoloTargetEffect());
	Template.AddTargetEffect(class'X2Ability_GrenadierAbilitySet'.static.ShredderDamageEffect());

	Template.bAllowAmmoEffects = true;
	
	
	
	RemoveEffects = new class'X2Effect_RemoveEffects';
	RemoveEffects.EffectNamesToRemove.AddItem('Isms_SoulCollector');
	RemoveEffects.EffectNamesToRemove.AddItem('RushOfBattleMax');
	RemoveEffects.bcleanse = true;  //if false = heal, if true = no heal.
	RemoveEffects.bApplyOnMiss = true;
 	Template.AddShooterEffect(RemoveEffects);

	ImpairingAbilityEffect = new class 'X2Effect_ImmediateAbilityActivation';
	ImpairingAbilityEffect.BuildPersistentEffect(1, false, true, , eGameRule_PlayerTurnBegin);
	ImpairingAbilityEffect.EffectName = 'IsmsNBK2';
	ImpairingAbilityEffect.AbilityName = 'IsmsNBK2';
	ImpairingAbilityEffect.bRemoveWhenTargetDies = true;
	Template.AddShooterEffect(ImpairingAbilityEffect);


 	Template.bShowActivation = true;
	//Template.CustomFireAnim = 'HL_Psi_SelfCast';
	Template.ActivationSpeech = 'CombatStim';
	//Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	Template.BuildVisualizationFn = TypicalAbility_BuildVisualization;
	Template.CinescriptCameraType = "StandardGunFiring";
	//Template.AdditionalAbilities.AddItem('IsmsSoulCollector2');
	//Template.AdditionalAbilities.AddItem('IsmsSoulCollector3');
	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;

	return Template;
}

  
	static function X2AbilityTemplate IsmsNBK2()
{
	local X2AbilityTemplate								Template;
	local X2AbilityTargetStyle							TargetStyle;
	local X2Effect_Isms_SoulCollector				TrackerEffect;
	local X2Effect_PersistentStatChange         PersistentStatChangeEffect;


	`CREATE_X2ABILITY_TEMPLATE(Template, 'IsmsNBK2');
	Template.eAbilityIconBehaviorHUD = eAbilityIconBehavior_NeverShow;	
	Template.IconImage = "img:///UILibrary_PerkIcons.UIPerk_executioner";
	Template.Hostility = eHostility_Neutral;
	Template.AbilitySourceName = 'eAbilitySource_Perk';
	
	Template.AbilityToHitCalc = default.DeadEye;

	TargetStyle = new class'X2AbilityTarget_Self';
	Template.AbilityTargetStyle = TargetStyle;
	
	Template.AbilityTriggers.AddItem(new class'X2AbilityTrigger_Placeholder');
  
 	TrackerEffect = new class'X2Effect_Isms_SoulCollector';
	TrackerEffect.EffectName = 'Isms_SoulCollector';
	TrackerEffect.BuildPersistentEffect(1, true, true, false, eGameRule_PlayerTurnBegin);
	TrackerEffect.SetDisplayInfo(ePerkBuff_Passive, Template.LocFriendlyName, Template.GetMyLongDescription(), Template.IconImage,,,Template.AbilitySourceName);
	TrackerEffect.DuplicateResponse = eDupe_ignore;
	Template.AddTargetEffect(TrackerEffect);
 //
  	PersistentStatChangeEffect = new class'X2Effect_PersistentStatChange';
	PersistentStatChangeEffect.EffectName = 'RushofBattleMax';
	PersistentStatChangeEffect.AddPersistentStatChange(eStat_Mobility, 6);
	//PersistentStatChangeEffect.AddPersistentStatChange(eStat_Dodge, 18);
	//PersistentStatChangeEffect.SetDisplayInfo(ePerkBuff_Passive, "TestThisShit", Template.GetMyLongDescription(), Template.IconImage, true,,Template.AbilitySourceName);
	PersistentStatChangeEffect.DuplicateREsponse = eDupe_Refresh;
	Template.AddTargetEffect(PersistentStatChangeEffect);


	Template.BuildNewGameStateFn = TypicalAbility_BuildGameState;
	return Template;
}
 