//---------------------------------------------------------------------------------------
//  FILE:    X2Effect_PersistentStatChange.uc
//  AUTHOR:  Timothy Talley
//           
//---------------------------------------------------------------------------------------
//  Copyright (c) 2016 Firaxis Games, Inc. All rights reserved.
//---------------------------------------------------------------------------------------
class X2Effect_IntelCharges extends X2Effect_ModifyStats;

var array<StatChange>	m_aStatChanges;

simulated function AddPersistentStatChange(ECharStatType StatType, float StatAmount, optional EStatModOp InModOp=MODOP_Addition )
{
	local StatChange NewChange;
	
	NewChange.StatType = StatType;
	NewChange.StatAmount = StatAmount;
	NewChange.ModOp = InModOp;

	m_aStatChanges.AddItem(NewChange);
}
 

simulated protected function OnEffectAdded(const out EffectAppliedData ApplyEffectParameters, XComGameState_BaseObject kNewTargetState, XComGameState NewGameState, XComGameState_Effect NewEffectState)
{
	local XComGameState_Unit EffectTargetUnit; 
	local XComGameStateHistory History;
	local StateObjectReference         AbilityRef;
	local XComGameState_Ability		   AbilityState;
	local int NumCharges;

	NewEffectState.StatChanges = m_aStatChanges;
	super.OnEffectAdded(ApplyEffectParameters, kNewTargetState, NewGameState, NewEffectState);

	History = `XCOMHISTORY;
	EffectTargetUnit = XComGameState_Unit(History.GetGameStateForObjectID(ApplyEffectParameters.TargetStateObjectRef.ObjectID));
 
 		
	AbilityRef = EffectTargetUnit.FindAbility('IsmsTactics');
	AbilityState = XComGameState_Ability(`XCOMHISTORY.GetGameStateForObjectID(AbilityRef.ObjectID));

	if(AbilityState != none && AbilityState.GetMyTemplateName() == 'IsmsTactics') 
		{
		NumCharges = AbilityState.iCharges;
		AbilityState.iCharges = NumCharges+1;
		}
 



}
 

simulated function OnEffectRemoved(const out EffectAppliedData ApplyEffectParameters, XComGameState NewGameState, bool bCleansed, XComGameState_Effect RemovedEffectState)
{
 

	local XComGameState_Unit TargetUnitState;
 	local XComGameStateHistory History;
	local StateObjectReference         AbilityRef;
	local XComGameState_Ability		   AbilityState;
	local int NumbCharges; 	

	super.OnEffectRemoved(ApplyEffectParameters, NewGameState, bCleansed, RemovedEffectState);
 
 	History = `XCOMHISTORY;
//	EffectTargetUnit = XComGameState_Unit(History.GetGameStateForObjectID(ApplyEffectParameters.TargetStateObjectRef.ObjectID));
	TargetUnitState = XComGameState_Unit(History.GetGameStateForObjectID(ApplyEffectParameters.TargetStateObjectRef.ObjectID));

 		
	AbilityRef = TargetUnitState.FindAbility('IsmsTactics');
	AbilityState = XComGameState_Ability(`XCOMHISTORY.GetGameStateForObjectID(AbilityRef.ObjectID));

	if(AbilityState != none && AbilityState.GetMyTemplateName() == 'IsmsTactics') 
		{
		NumbCharges = AbilityState.iCharges;
		AbilityState.iCharges = NumbCharges - 1;

		} 
 
	if (bCleansed)
		{	
	
	AbilityRef = TargetUnitState.FindAbility('IsmsTactics');
	AbilityState = XComGameState_Ability(`XCOMHISTORY.GetGameStateForObjectID(AbilityRef.ObjectID));

	if(AbilityState != none && AbilityState.GetMyTemplateName() == 'IsmsTactics') 
		{ 
		AbilityState.iCharges = 0;

		} 		
		} 
 

}
 