//---------------------------------------------------------------------------------------
//  FILE:    X2Effect_PersistentStatChange.uc
//  AUTHOR:  Timothy Talley
//           
//---------------------------------------------------------------------------------------
//  Copyright (c) 2016 Firaxis Games, Inc. All rights reserved.
//---------------------------------------------------------------------------------------
class X2Effect_NBKMobility extends X2Effect_ModifyStats;

var array<StatChange>	m_aStatChanges;

simulated function AddPersistentStatChange(ECharStatType StatType, float StatAmount, optional EStatModOp InModOp=MODOP_Addition )
{
	local StatChange NewChange;
	
	NewChange.StatType = StatType;
	NewChange.StatAmount = StatAmount;
	NewChange.ModOp = InModOp;

	m_aStatChanges.AddItem(NewChange);
}

simulated protected function OnEffectAdded(const out EffectAppliedData ApplyEffectParameters, XComGameState_BaseObject kNewTargetState, XComGameState NewGameState, XComGameState_Effect NewEffectState)
{
	local XComGameState_Unit EffectTargetUnit; 
	local XComGameStateHistory History;
	local StateObjectReference         AbilityRef;
	local XComGameState_Ability		   AbilityState;
	local int NumCharges;

	NewEffectState.StatChanges = m_aStatChanges;
	super.OnEffectAdded(ApplyEffectParameters, kNewTargetState, NewGameState, NewEffectState);

	History = `XCOMHISTORY;
	EffectTargetUnit = XComGameState_Unit(History.GetGameStateForObjectID(ApplyEffectParameters.TargetStateObjectRef.ObjectID));
 
 		
	AbilityRef = EffectTargetUnit.FindAbility('IsmsNBKGrandFinale');
	AbilityState = XComGameState_Ability(`XCOMHISTORY.GetGameStateForObjectID(AbilityRef.ObjectID));

	if(AbilityState != none && AbilityState.GetMyTemplateName() == 'IsmsNBKGrandFinale') 
		{
		NumCharges = AbilityState.iCharges;
		AbilityState.iCharges = NumCharges+1;
		}
 



}
 

simulated function OnEffectRemoved(const out EffectAppliedData ApplyEffectParameters, XComGameState NewGameState, bool bCleansed, XComGameState_Effect RemovedEffectState)
{
 

	local XComGameState_Unit TargetUnitState;
 	local XComGameStateHistory History;
	local StateObjectReference         AbilityRef;
	local XComGameState_Ability		   AbilityState;
	local int NumbCharges; 	

	super.OnEffectRemoved(ApplyEffectParameters, NewGameState, bCleansed, RemovedEffectState);
 
 	History = `XCOMHISTORY;
//	EffectTargetUnit = XComGameState_Unit(History.GetGameStateForObjectID(ApplyEffectParameters.TargetStateObjectRef.ObjectID));
	TargetUnitState = XComGameState_Unit(History.GetGameStateForObjectID(ApplyEffectParameters.TargetStateObjectRef.ObjectID));

 		
	AbilityRef = TargetUnitState.FindAbility('IsmsNBKGrandFinale');
	AbilityState = XComGameState_Ability(`XCOMHISTORY.GetGameStateForObjectID(AbilityRef.ObjectID));

	if(AbilityState != none && AbilityState.GetMyTemplateName() == 'IsmsNBKGrandFinale') 
		{
		NumbCharges = AbilityState.iCharges;
		AbilityState.iCharges = NumbCharges - 1;

		} 
 
	//if (bCleansed)
		//{	
	//
	//AbilityRef = TargetUnitState.FindAbility('IsmsNBKGrandFinale');
	//AbilityState = XComGameState_Ability(`XCOMHISTORY.GetGameStateForObjectID(AbilityRef.ObjectID));
//
	//if(AbilityState != none && AbilityState.GetMyTemplateName() == 'IsmsNBKGrandFinale') 
		//{ 
		//AbilityState.iCharges = 0;
//
		//} 		
		//} 
 

}

function int GetAttackingDamageModifier(XComGameState_Effect EffectState, XComGameState_Unit Attacker, Damageable TargetDamageable, XComGameState_Ability AbilityState, const out EffectAppliedData AppliedData, const int CurrentDamage, optional XComGameState NewGameState)
{
	local XComGameState_Unit TargetUnit;
	local GameRulesCache_VisibilityInfo VisInfo;
	local float ExtraDamage;
	local XComGameState_Item SourceWeapon;  ///Here Delete
	//local int NumCharges; 
//
//
	//NumCharges = AbilityState.iCharges;

		if (AbilityState.GetMyTemplateName() == 'IsmsNBKGrandFinale')
	{
		 
		SourceWeapon = AbilityState.GetSourceWeapon();		
		 
		ExtraDamage = 1;  //this adds +1 per stack of NBK, or per charge, rather.
 


	TargetUnit = XComGameState_Unit(TargetDamageable);
		If (SourceWeapon.GetWeaponCategory() != 'grenade')
	{
	If (CurrentDamage > 1)
	{
	if (!AbilityState.IsMeleeAbility() && TargetUnit != None && class'XComGameStateContext_Ability'.static.IsHitResultHit(AppliedData.AbilityResultContext.HitResult))
	{
		if (`TACTICALRULES.VisibilityMgr.GetVisibilityInfo(Attacker.ObjectID, TargetUnit.ObjectID, VisInfo))
		{
				if (Attacker.CanFlank() && TargetUnit.CanTakeCover() && VisInfo.TargetCover == CT_None)
			{
				return ExtraDamage;
			}
			if (Attacker.CanFlank() && !TargetUnit.CanTakeCover() && VisInfo.TargetCover == CT_None)
			{
				if (TargetUnit.GetMaxStat(eStat_HP) >= 12) 
			{
				return ExtraDamage;
			}			
				else if (TargetUnit.GetMaxStat(eStat_HP) <= 12) 
			{
				return ExtraDamage;
			}							
			}
		}
	}
	}
	}
	return 0;

	}
}