
class X2Effect_RogueCreateMarkedEffect extends X2StatusEffects config(GameCore);

static function X2Effect_Persistent CreateMarkedEffect(int NumTurns, bool bIsInfinite)
{
	local X2Effect_Persistent PersistentEffect;

	PersistentEffect = new class 'X2Effect_RogueMarked';
	PersistentEffect.EffectName = 'Scouted';
	PersistentEffect.DuplicateResponse = eDupe_Ignore;
	PersistentEffect.BuildPersistentEffect(NumTurns, bIsInfinite, true,,eGameRule_PlayerTurnEnd);
	PersistentEffect.SetDisplayInfo(ePerkBuff_Penalty, "Scouted", "Scouted", "img:///UILibrary_PerkIcons.UIPerk_mark");
	PersistentEffect.VisualizationFn = ScoutedVisualization;
	PersistentEffect.EffectTickedVisualizationFn = MarkedVisualizationTicked;
	PersistentEffect.EffectRemovedVisualizationFn = MarkedVisualizationRemoved;
	PersistentEffect.bRemoveWhenTargetDies = true;

	return PersistentEffect;
}

static function X2Effect_Persistent CreateTauntEffect(int NumTurns, bool bIsInfinite)
{
	local X2Effect_Persistent PersistentEffect;

	PersistentEffect = new class 'X2Effect_RogueTaunt';
	PersistentEffect.EffectName = 'Taunt';
	PersistentEffect.DuplicateResponse = eDupe_Ignore;
	PersistentEffect.BuildPersistentEffect(NumTurns, bIsInfinite, true,,eGameRule_PlayerTurnEnd);
	PersistentEffect.SetDisplayInfo(ePerkBuff_Bonus, "Taunting", "Taunting", "img:///UILibrary_PerkIcons.UIPerk_Riposte");
	PersistentEffect.VisualizationFn = MarkedVisualization;
	PersistentEffect.EffectTickedVisualizationFn = MarkedVisualizationTicked;
	PersistentEffect.EffectRemovedVisualizationFn = MarkedVisualizationRemoved;
	PersistentEffect.bRemoveWhenTargetDies = true;

	return PersistentEffect;
}

static function X2Effect_Persistent CreateReducedThreatEffect(int NumTurns, bool bIsInfinite)
{
	local X2Effect_Persistent PersistentEffect;

	PersistentEffect = new class 'X2Effect_ReducedThreat';
	PersistentEffect.EffectName = 'ReducedThreat';
	PersistentEffect.DuplicateResponse = eDupe_Ignore;
	PersistentEffect.BuildPersistentEffect(NumTurns, bIsInfinite, true,,eGameRule_PlayerTurnEnd);
	PersistentEffect.SetDisplayInfo(ePerkBuff_Bonus, "Taunting", "Taunting", "img:///UILibrary_PerkIcons.UIPerk_Riposte");
	PersistentEffect.VisualizationFn = MarkedVisualization;
	PersistentEffect.EffectTickedVisualizationFn = MarkedVisualizationTicked;
	PersistentEffect.EffectRemovedVisualizationFn = MarkedVisualizationRemoved;
	PersistentEffect.bRemoveWhenTargetDies = true;

	return PersistentEffect;
}

static function X2Effect_Persistent CreateRelocateStealthEffect(int NumTurns, bool bIsInfinite)
{
	local X2Effect_Persistent PersistentEffect;

	PersistentEffect = new class 'X2Effect_Isms_RelocateRemove';
	PersistentEffect.EffectName = 'RelocateStealth';
	PersistentEffect.DuplicateResponse = eDupe_Ignore;
	PersistentEffect.BuildPersistentEffect(NumTurns, bIsInfinite, true,,eGameRule_PlayerTurnBegin);
	PersistentEffect.SetDisplayInfo(ePerkBuff_Bonus, "Invisible", "Invisible", "img:///UILibrary_PerkIcons.UIPerk_Riposte");
	//PersistentEffect.VisualizationFn = MarkedVisualization;
	//PersistentEffect.EffectTickedVisualizationFn = MarkedVisualizationTicked;
	//PersistentEffect.EffectRemovedVisualizationFn = MarkedVisualizationRemoved;
	PersistentEffect.bRemoveWhenTargetDies = true;

	return PersistentEffect;
}


static function X2Effect_Persistent CreateParriedEffect(int NumTurns, bool bIsInfinite)
{
	local X2Effect_Persistent PersistentEffect;

	PersistentEffect = new class 'X2Effect_Parried';
	PersistentEffect.EffectName = 'Parried';
	PersistentEffect.DuplicateResponse = eDupe_Ignore;
	PersistentEffect.BuildPersistentEffect(NumTurns, bIsInfinite, true,,eGameRule_PlayerTurnEnd);
	PersistentEffect.SetDisplayInfo(ePerkBuff_Penalty, "Parried", "Parried", "img:///UILibrary_PerkIcons.UIPerk_mark");
	PersistentEffect.VisualizationFn = ParriedVisualization;
	PersistentEffect.EffectTickedVisualizationFn = MarkedVisualizationTicked;
	PersistentEffect.EffectRemovedVisualizationFn = MarkedVisualizationRemoved;
	PersistentEffect.bRemoveWhenTargetDies = true;

	return PersistentEffect;
}

static function MarkedVisualization(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	if( EffectApplyResult != 'AA_Success' )
	{
		return;
	}
	if (XComGameState_Unit(BuildTrack.StateObject_NewState) == none)
		return;

	AddEffectSoundAndFlyOverToTrack(BuildTrack, VisualizeGameState.GetContext(), default.MarkedFriendlyName, '', eColor_Bad, class'UIUtilities_Image'.const.UnitStatus_Marked);
	AddEffectMessageToTrack(BuildTrack, default.MarkedEffectAcquiredString, VisualizeGameState.GetContext());
	UpdateUnitFlag(BuildTrack, VisualizeGameState.GetContext());
}

static function MarkedVisualizationTicked(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	local XComGameState_Unit UnitState;

	UnitState = XComGameState_Unit(BuildTrack.StateObject_NewState);
	if (UnitState == none)
		return;

	// dead units should not be reported
	if( !UnitState.IsAlive() )
	{
		return;
	}

	AddEffectSoundAndFlyOverToTrack(BuildTrack, VisualizeGameState.GetContext(), default.MarkedFriendlyName, '', eColor_Bad, class'UIUtilities_Image'.const.UnitStatus_Marked);
	AddEffectMessageToTrack(BuildTrack, default.MarkedEffectTickedString, VisualizeGameState.GetContext());
	UpdateUnitFlag(BuildTrack, VisualizeGameState.GetContext());
}






static function MarkedVisualizationRemoved(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	local XComGameState_Unit UnitState;

	UnitState = XComGameState_Unit(BuildTrack.StateObject_NewState);
	if (UnitState == none)
		return;

	// dead units should not be reported
	if( !UnitState.IsAlive() )
	{
		return;
	}

	AddEffectMessageToTrack(BuildTrack, default.MarkedEffectLostString, VisualizeGameState.GetContext());
	UpdateUnitFlag(BuildTrack, VisualizeGameState.GetContext());
}

static function ParriedVisualization(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	if( EffectApplyResult != 'AA_Success' )
	{
		return;
	}
	if (XComGameState_Unit(BuildTrack.StateObject_NewState) == none)
		return;

	AddEffectSoundAndFlyOverToTrack(BuildTrack, VisualizeGameState.GetContext(), "Parried", '', eColor_Bad, class'UIUtilities_Image'.const.UnitStatus_Marked);
	AddEffectMessageToTrack(BuildTrack, "Parried", VisualizeGameState.GetContext());
	UpdateUnitFlag(BuildTrack, VisualizeGameState.GetContext());
}

static function ScoutedVisualization(XComGameState VisualizeGameState, out VisualizationTrack BuildTrack, const name EffectApplyResult)
{
	if( EffectApplyResult != 'AA_Success' )
	{
		return;
	}
	if (XComGameState_Unit(BuildTrack.StateObject_NewState) == none)
		return;

	AddEffectSoundAndFlyOverToTrack(BuildTrack, VisualizeGameState.GetContext(), "Scouted", '', eColor_Bad, class'UIUtilities_Image'.const.UnitStatus_Marked);
	AddEffectMessageToTrack(BuildTrack, "Scouted", VisualizeGameState.GetContext());
	UpdateUnitFlag(BuildTrack, VisualizeGameState.GetContext());
}